@@ -80,7 +80,7 @@ TALER_BANK_make_auth_header_ (const struct TALER_BANK_AuthenticationData *auth)
     authh = append (authh,
                     "X-Taler-Bank-Password",
                     auth->details.basic.password);
-    return authh;    
+    return authh;
   }
   return NULL;
 }
@@ -91,7 +91,7 @@ TALER_BANK_make_auth_header_ (const struct TALER_BANK_AuthenticationData *auth)
  *
  * @param u base URL of the bank
  * @param path Taler API path (i.e. "/history")
- * @return the full URI to use with cURL
+ * @return the full URL to use with cURL
  */
 char *
 TALER_BANK_path_to_url_ (const char *u,
@@ -111,5 +111,33 @@ TALER_BANK_path_to_url_ (const char *u,
 }
 
 
+/**
+ * Parse error code given in @a json.
+ *
+ * @param json the json to parse
+ * @return error code, or #TALER_EC_INVALID if not found
+ */
+enum TALER_ErrorCode
+TALER_BANK_parse_ec_ (const json_t *json)
+{
+  uint32_t ec;
+
+  struct GNUNET_JSON_Specification spec[] = {
+    GNUNET_JSON_spec_uint32 ("ec",
+                             &ec),
+    GNUNET_JSON_spec_end()
+  };
+
+  if (GNUNET_OK !=
+      GNUNET_JSON_parse (json,
+                         spec,
+                         NULL, NULL))
+  {
+    GNUNET_break_op (0);
+    return TALER_EC_INVALID;
+  }
+  return (enum TALER_ErrorCode) ec;
+}
+
 
 /* end of bank_api_common.c */
