@@ -80,7 +80,19 @@ struct Transaction
   /**
    * Number of this transaction.
    */
-  uint64_t serial_id;
+  uint64_t row_id;
+
+  /**
+   * Flag set if the transfer was rejected.
+   */
+  int rejected;
+
+  /**
+   * Has this transaction been subjected to #TALER_FAKEBANK_check()
+   * and should thus no longer be counted in
+   * #TALER_FAKEBANK_check_empty()?
+   */
+  int checked;
 };
 
 
@@ -145,15 +157,12 @@ TALER_FAKEBANK_check (struct TALER_FAKEBANK_Handle *h,
          (want_credit == t->credit_account) &&
          (0 == TALER_amount_cmp (want_amount,
                                  &t->amount)) &&
+         (GNUNET_NO == t->checked) &&
          (0 == strcasecmp (exchange_base_url,
                            t->exchange_base_url)) )
     {
-      GNUNET_CONTAINER_DLL_remove (h->transactions_head,
-                                   h->transactions_tail,
-                                   t);
-      *subject = t->subject;
-      GNUNET_free (t->exchange_base_url);
-      GNUNET_free (t);
+      *subject = GNUNET_strdup (t->subject);
+      t->checked = GNUNET_YES;
       return GNUNET_OK;
     }
   }
@@ -161,17 +170,21 @@ TALER_FAKEBANK_check (struct TALER_FAKEBANK_Handle *h,
            "Did not find matching transaction!\nI have:\n");
   for (struct Transaction *t = h->transactions_head; NULL != t; t = t->next)
   {
-    char *s;
-
-    s = TALER_amount_to_string (&t->amount);
+    if (GNUNET_YES == t->checked)
+      continue;
     fprintf (stderr,
              "%llu -> %llu (%s) from %s\n",
              (unsigned long long) t->debit_account,
              (unsigned long long) t->credit_account,
-             s,
+             TALER_amount2s (&t->amount),
              t->exchange_base_url);
-    GNUNET_free (s);
   }
+  fprintf (stderr,
+           "I wanted:\n%llu -> %llu (%s) from %s\n",
+           (unsigned long long) want_debit,
+           (unsigned long long) want_credit,
+           TALER_amount2s (want_amount),
+           exchange_base_url);
   return GNUNET_SYSERR;
 }
 
@@ -185,7 +198,7 @@ TALER_FAKEBANK_check (struct TALER_FAKEBANK_Handle *h,
  * @param amount amount to transfer
  * @param subject wire transfer subject to use
  * @param exchange_base_url exchange URL
- * @return serial_id of the transfer
+ * @return row_id of the transfer
  */
 uint64_t
 TALER_FAKEBANK_make_transfer (struct TALER_FAKEBANK_Handle *h,
@@ -208,14 +221,39 @@ TALER_FAKEBANK_make_transfer (struct TALER_FAKEBANK_Handle *h,
   t->credit_account = credit_account;
   t->amount = *amount;
   t->exchange_base_url = GNUNET_strdup (exchange_base_url);
-  t->serial_id = ++h->serial_counter;
+  t->row_id = ++h->serial_counter;
   t->date = GNUNET_TIME_absolute_get ();
   t->subject = GNUNET_strdup (subject);
   GNUNET_TIME_round_abs (&t->date);
   GNUNET_CONTAINER_DLL_insert_tail (h->transactions_head,
                                     h->transactions_tail,
                                     t);
-  return t->serial_id;
+  return t->row_id;
+}
+
+
+/**
+ * Reject incoming wire transfer to account @a credit_account
+ * as identified by @a rowid.
+ *
+ * @param h fake bank handle
+ * @param rowid identifies transfer to reject
+ * @param credit_account account number of owner of credited account
+ * @return #GNUNET_YES on success, #GNUNET_NO if the wire transfer was not found
+ */
+int
+TALER_FAKEBANK_reject_transfer (struct TALER_FAKEBANK_Handle *h,
+                                uint64_t rowid,
+                                uint64_t credit_account)
+{
+  for (struct Transaction *t = h->transactions_head; NULL != t; t = t->next)
+    if ( (t->row_id == rowid) &&
+         (t->credit_account == credit_account) )
+    {
+      t->rejected = GNUNET_YES;
+      return GNUNET_YES;
+    }
+  return GNUNET_NO;
 }
 
 
@@ -232,23 +270,35 @@ TALER_FAKEBANK_check_empty (struct TALER_FAKEBANK_Handle *h)
 {
   struct Transaction *t;
 
-  if (NULL == h->transactions_head)
+  t = h->transactions_head;
+  while (NULL != t)
+  {
+    if ( (GNUNET_YES != t->checked) &&
+         (GNUNET_YES != t->rejected) )
+      break;
+    t = t->next;
+  }
+  if (NULL == t)
     return GNUNET_OK;
-
   fprintf (stderr,
            "Expected empty transaction set, but I have:\n");
-  for (t = h->transactions_head; NULL != t; t = t->next)
+  while (NULL != t)
   {
-    char *s;
+    if ( (GNUNET_YES != t->checked) &&
+         (GNUNET_YES != t->rejected) )
+    {
+      char *s;
 
-    s = TALER_amount_to_string (&t->amount);
-    fprintf (stderr,
-             "%llu -> %llu (%s) from %s\n",
-             (unsigned long long) t->debit_account,
-             (unsigned long long) t->credit_account,
-             s,
-             t->exchange_base_url);
-    GNUNET_free (s);
+      s = TALER_amount_to_string (&t->amount);
+      fprintf (stderr,
+               "%llu -> %llu (%s) from %s\n",
+               (unsigned long long) t->debit_account,
+               (unsigned long long) t->credit_account,
+               s,
+               t->exchange_base_url);
+      GNUNET_free (s);
+    }
+    t = t->next;
   }
   return GNUNET_SYSERR;
 }
@@ -287,6 +337,62 @@ TALER_FAKEBANK_stop (struct TALER_FAKEBANK_Handle *h)
 }
 
 
+/**
+ * Create and queue a bank error message with the HTTP response
+ * code @a response_code on connection @a connection.
+ *
+ * @param connection where to queue the reply
+ * @param response_code http status code to use
+ * @param ec taler error code to use
+ * @param message human readable error message
+ * @return MHD status code
+ */
+static int
+create_bank_error (struct MHD_Connection *connection,
+                   unsigned int response_code,
+                   enum TALER_ErrorCode ec,
+                   const char *message)
+{
+  json_t *json;
+  struct MHD_Response *resp;
+  void *json_str;
+  size_t json_len;
+  int ret;
+
+  json = json_pack ("{s:s, s:I}",
+                    "error",
+                    message,
+                    "ec",
+                    (json_int_t) ec);
+  json_str = json_dumps (json,
+                         JSON_INDENT(2));
+  json_decref (json);
+  if (NULL == json_str)
+  {
+    GNUNET_break (0);
+    return MHD_NO;
+  }
+  json_len = strlen (json_str);
+  resp = MHD_create_response_from_buffer (json_len,
+                                          json_str,
+                                          MHD_RESPMEM_MUST_FREE);
+  if (NULL == resp)
+  {
+    GNUNET_break (0);
+    free (json_str);
+    return MHD_NO;
+  }
+  (void) MHD_add_response_header (resp,
+                                  MHD_HTTP_HEADER_CONTENT_TYPE,
+                                  "application/json");
+  ret = MHD_queue_response (connection,
+                            response_code,
+                            resp);
+  MHD_destroy_response (resp);
+  return ret;
+}
+
+
 /**
  * Function called whenever MHD is done with a request.  If the
  * request was a POST, we may have stored a `struct Buffer *` in the
@@ -335,7 +441,7 @@ handle_admin_add_incoming (struct TALER_FAKEBANK_Handle *h,
   json_t *json;
   struct MHD_Response *resp;
   int ret;
-  uint64_t serial_id;
+  uint64_t row_id;
 
   pr = GNUNET_JSON_post_parser (REQUEST_BUFFER_MAX,
                                 con_cls,
@@ -359,13 +465,13 @@ handle_admin_add_incoming (struct TALER_FAKEBANK_Handle *h,
     break;
   }
   {
-    const char *wtid;
+    const char *subject;
     uint64_t debit_account;
     uint64_t credit_account;
     const char *base_url;
     struct TALER_Amount amount;
     struct GNUNET_JSON_Specification spec[] = {
-      GNUNET_JSON_spec_string ("wtid", &wtid),
+      GNUNET_JSON_spec_string ("subject", &subject),
       GNUNET_JSON_spec_uint64 ("debit_account", &debit_account),
       GNUNET_JSON_spec_uint64 ("credit_account", &credit_account),
       TALER_JSON_spec_amount ("amount", &amount),
@@ -381,11 +487,11 @@ handle_admin_add_incoming (struct TALER_FAKEBANK_Handle *h,
       json_decref (json);
       return MHD_NO;
     }
-    serial_id = TALER_FAKEBANK_make_transfer (h,
+    row_id = TALER_FAKEBANK_make_transfer (h,
                                               debit_account,
                                               credit_account,
                                               &amount,
-                                              wtid,
+                                              subject,
                                               base_url);
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Receiving incoming wire transfer: %llu->%llu from %s\n",
@@ -401,8 +507,8 @@ handle_admin_add_incoming (struct TALER_FAKEBANK_Handle *h,
     size_t json_len;
 
     json = json_pack ("{s:I}",
-                      "serial_id",
-                      (json_int_t) serial_id);
+                      "row_id",
+                      (json_int_t) row_id);
     json_str = json_dumps (json,
                            JSON_INDENT(2));
     json_decref (json);
@@ -433,6 +539,94 @@ handle_admin_add_incoming (struct TALER_FAKEBANK_Handle *h,
 }
 
 
+/**
+ * Handle incoming HTTP request for /reject.
+ *
+ * @param h the fakebank handle
+ * @param connection the connection
+ * @param upload_data request data
+ * @param upload_data_size size of @a upload_data in bytes
+ * @param con_cls closure for request (a `struct Buffer *`)
+ * @return MHD result code
+ */
+static int
+handle_reject (struct TALER_FAKEBANK_Handle *h,
+               struct MHD_Connection *connection,
+               const char *upload_data,
+               size_t *upload_data_size,
+               void **con_cls)
+{
+  enum GNUNET_JSON_PostResult pr;
+  json_t *json;
+  struct MHD_Response *resp;
+  int ret;
+  int found;
+
+  pr = GNUNET_JSON_post_parser (REQUEST_BUFFER_MAX,
+                                con_cls,
+                                upload_data,
+                                upload_data_size,
+                                &json);
+  switch (pr)
+  {
+  case GNUNET_JSON_PR_OUT_OF_MEMORY:
+    GNUNET_break (0);
+    return MHD_NO;
+  case GNUNET_JSON_PR_CONTINUE:
+    return MHD_YES;
+  case GNUNET_JSON_PR_REQUEST_TOO_LARGE:
+    GNUNET_break (0);
+    return MHD_NO;
+  case GNUNET_JSON_PR_JSON_INVALID:
+    GNUNET_break (0);
+    return MHD_NO;
+  case GNUNET_JSON_PR_SUCCESS:
+    break;
+  }
+  {
+    uint64_t row_id;
+    uint64_t credit_account;
+    struct GNUNET_JSON_Specification spec[] = {
+      GNUNET_JSON_spec_uint64 ("row_id", &row_id),
+      GNUNET_JSON_spec_uint64 ("account_number", &credit_account),
+      GNUNET_JSON_spec_end ()
+    };
+    if (GNUNET_OK !=
+        GNUNET_JSON_parse (json,
+                           spec,
+                           NULL, NULL))
+    {
+      GNUNET_break (0);
+      json_decref (json);
+      return MHD_NO;
+    }
+    found = TALER_FAKEBANK_reject_transfer (h,
+                                            row_id,
+                                            credit_account);
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Rejected wire transfer #%llu (to %llu)\n",
+                (unsigned long long) row_id,
+                (unsigned long long) credit_account);
+  }
+  json_decref (json);
+
+  if (GNUNET_OK != found)
+    return create_bank_error (connection,
+                              MHD_HTTP_NOT_FOUND,
+                              TALER_EC_BANK_REJECT_TRANSACTION_NOT_FOUND,
+                              "transaction unknown");
+  /* finally build regular response */
+  resp = MHD_create_response_from_buffer (0,
+                                          NULL,
+                                          MHD_RESPMEM_PERSISTENT);
+  ret = MHD_queue_response (connection,
+                            MHD_HTTP_NO_CONTENT,
+                            resp);
+  MHD_destroy_response (resp);
+  return ret;
+}
+
+
 /**
  * Handle incoming HTTP request for /history
  *
@@ -451,6 +645,7 @@ handle_history (struct TALER_FAKEBANK_Handle *h,
   const char *start;
   const char *dir;
   const char *acc;
+  const char *cancelled;
   unsigned long long account_number;
   unsigned long long start_number;
   long long count;
@@ -469,6 +664,9 @@ handle_history (struct TALER_FAKEBANK_Handle *h,
   dir = MHD_lookup_connection_value (connection,
                                      MHD_GET_ARGUMENT_KIND,
                                      "direction");
+  cancelled = MHD_lookup_connection_value (connection,
+                                           MHD_GET_ARGUMENT_KIND,
+                                           "cancelled");
   start = MHD_lookup_connection_value (connection,
                                        MHD_GET_ARGUMENT_KIND,
                                        "start");
@@ -496,7 +694,14 @@ handle_history (struct TALER_FAKEBANK_Handle *h,
          (1 != sscanf (start,
                        "%llu",
                        &start_number)) ) ||
-       ( (NULL != dir) &&
+       (NULL == dir) ||
+       (NULL == cancelled) ||
+       ( (0 != strcasecmp (cancelled,
+                           "OMIT")) &&
+         (0 != strcasecmp (cancelled,
+                           "SHOW")) ) ||
+       ( (0 != strcasecmp (dir,
+                           "BOTH")) &&
          (0 != strcasecmp (dir,
                            "CREDIT")) &&
          (0 != strcasecmp (dir,
@@ -513,13 +718,40 @@ handle_history (struct TALER_FAKEBANK_Handle *h,
               dir,
               (unsigned long long) account_number,
               start);
-  if (NULL == dir)
-    direction = TALER_BANK_DIRECTION_BOTH;
-  else if (0 == strcasecmp (dir,
-                            "CREDIT"))
+  if (0 == strcasecmp (dir,
+                       "CREDIT"))
+  {
     direction = TALER_BANK_DIRECTION_CREDIT;
-  else
+  }
+  else if (0 == strcasecmp (dir,
+                            "DEBIT"))
+  {
     direction = TALER_BANK_DIRECTION_DEBIT;
+  }
+  else if (0 == strcasecmp (dir,
+                            "BOTH"))
+  {
+    direction = TALER_BANK_DIRECTION_BOTH;
+  }
+  else
+  {
+    GNUNET_assert (0);
+    return MHD_NO;
+  }
+  if (0 == strcasecmp (cancelled,
+                       "OMIT"))
+  {
+    /* nothing */
+  } else if (0 == strcasecmp (cancelled,
+                              "SHOW"))
+  {
+    direction |= TALER_BANK_DIRECTION_CANCEL;
+  }
+  else
+  {
+    GNUNET_assert (0);
+    return MHD_NO;
+  }
   if (NULL == start)
   {
     if (count > 0)
@@ -532,11 +764,13 @@ handle_history (struct TALER_FAKEBANK_Handle *h,
     for (pos = h->transactions_head;
          NULL != pos;
          pos = pos->next)
-      if (pos->serial_id == start_number)
+      if (pos->row_id  == start_number)
         break;
     if (NULL == pos)
     {
-      GNUNET_break (0);
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Invalid range specified, transaction %llu not known!\n",
+                  (unsigned long long) start_number);
       return MHD_NO;
     }
     /* range is exclusive, skip the matching entry */
@@ -557,6 +791,7 @@ handle_history (struct TALER_FAKEBANK_Handle *h,
   {
     json_t *trans;
     char *subject;
+    const char *sign;
 
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Found transaction over %s from %llu to %llu\n",
@@ -564,10 +799,12 @@ handle_history (struct TALER_FAKEBANK_Handle *h,
                 (unsigned long long) pos->debit_account,
                 (unsigned long long) pos->credit_account);
 
-    if (! ( ( (account_number == pos->debit_account) &&
-              (0 != (direction & TALER_BANK_DIRECTION_DEBIT)) ) ||
-            ( (account_number == pos->credit_account) &&
-              (0 != (direction & TALER_BANK_DIRECTION_CREDIT) ) ) ) )
+    if ( (! ( ( (account_number == pos->debit_account) &&
+                (0 != (direction & TALER_BANK_DIRECTION_DEBIT)) ) ||
+              ( (account_number == pos->credit_account) &&
+                (0 != (direction & TALER_BANK_DIRECTION_CREDIT) ) ) ) ) ||
+         ( (0 == (direction & TALER_BANK_DIRECTION_CANCEL)) &&
+           (GNUNET_YES == pos->rejected) ) )
     {
       if (count > 0)
         pos = pos->next;
@@ -580,18 +817,23 @@ handle_history (struct TALER_FAKEBANK_Handle *h,
                      "%s %s",
                      pos->subject,
                      pos->exchange_base_url);
+    sign =
+      (account_number == pos->debit_account)
+      ? (pos->rejected ? "cancel-" : "-")
+      : (pos->rejected ? "cancel+" : "+");
     trans = json_pack ("{s:I, s:o, s:o, s:s, s:I, s:s}",
-                       "row_id", (json_int_t) pos->serial_id,
+                       "row_id", (json_int_t) pos->row_id,
                        "date", GNUNET_JSON_from_time_abs (pos->date),
                        "amount", TALER_JSON_from_amount (&pos->amount),
-                       "sign", (account_number == pos->debit_account) ? "-" : "+",
+                       "sign", sign,
                        "counterpart", (json_int_t) ( (account_number == pos->debit_account)
                                                      ? pos->credit_account
                                                      : pos->debit_account),
                        "wt_subject", subject);
+    GNUNET_assert (NULL != trans);
     GNUNET_free (subject);
-    json_array_append (history,
-                       trans);
+    GNUNET_assert (0 == json_array_append_new (history,
+                                               trans));
     if (count > 0)
     {
       pos = pos->next;
@@ -621,7 +863,8 @@ handle_history (struct TALER_FAKEBANK_Handle *h,
   }
 
   jresponse = json_pack ("{s:o}",
-                         "data", history);
+                         "data",
+                         history);
   if (NULL == jresponse)
   {
     GNUNET_break (0);
@@ -698,6 +941,15 @@ handle_mhd_request (void *cls,
                                       upload_data,
                                       upload_data_size,
                                       con_cls);
+  if ( (0 == strcasecmp (url,
+                         "/reject")) &&
+       (0 == strcasecmp (method,
+                         MHD_HTTP_METHOD_POST)) )
+    return handle_reject (h,
+                          connection,
+                          upload_data,
+                          upload_data_size,
+                          con_cls);
   if ( (0 == strcasecmp (url,
                          "/history")) &&
        (0 == strcasecmp (method,
@@ -706,7 +958,7 @@ handle_mhd_request (void *cls,
                            connection,
                            con_cls);
 
-  /* Unexpected URI path, just close the connection. */
+  /* Unexpected URL path, just close the connection. */
   /* we're rather impolite here, but it's a testcase. */
   GNUNET_break_op (0);
   return MHD_NO;
@@ -809,7 +1061,7 @@ TALER_FAKEBANK_start (uint16_t port)
   struct TALER_FAKEBANK_Handle *h;
 
   h = GNUNET_new (struct TALER_FAKEBANK_Handle);
-  h->mhd_bank = MHD_start_daemon (MHD_USE_DEBUG,
+  h->mhd_bank = MHD_start_daemon (MHD_USE_DEBUG | MHD_USE_DUAL_STACK,
                                   port,
                                   NULL, NULL,
                                   &handle_mhd_request, h,
