@@ -110,7 +110,6 @@ static const struct TBI_Command *
 find_command (const struct InterpreterState *is,
               const char *label)
 {
-  unsigned int i;
   const struct TBI_Command *cmd;
 
   if (NULL == label)
@@ -119,7 +118,7 @@ find_command (const struct InterpreterState *is,
                 "Attempt to lookup command for empty label\n");
     return NULL;
   }
-  for (i=0;TBI_OC_END != (cmd = &is->commands[i])->oc;i++)
+  for (unsigned int i=0;TBI_OC_END != (cmd = &is->commands[i])->oc;i++)
     if ( (NULL != cmd->label) &&
          (0 == strcmp (cmd->label,
                        label)) )
@@ -131,6 +130,34 @@ find_command (const struct InterpreterState *is,
 }
 
 
+/**
+ * Test if the /admin/add/incoming transaction at offset @a off
+ * has been /rejected.
+ *
+ * @param is interpreter state (where we are right now)
+ * @param off offset of the command to test for rejection
+ * @return #GNUNET_YES if the command at @a off was cancelled
+ */
+static int
+test_cancelled (struct InterpreterState *is,
+                unsigned int off)
+{
+  const struct TBI_Command *cmd = &is->commands[off];
+
+  for (unsigned int i=0;i<is->ip;i++)
+  {
+    const struct TBI_Command *c = &is->commands[i];
+
+    if (TBI_OC_REJECT != c->oc)
+      continue;
+    if (0 == strcmp (c->details.reject.cmd_ref,
+                     cmd->label))
+      return GNUNET_YES;
+  }
+  return GNUNET_NO;
+}
+
+
 /**
  * Item in the transaction history, as reconstructed from the
  * command history.
@@ -146,7 +173,7 @@ struct History
   /**
    * Serial ID of the wire transfer.
    */
-  uint64_t serial_id;
+  uint64_t row_id;
 
   /**
    * Direction of the transfer.
@@ -214,12 +241,13 @@ build_history (struct InterpreterState *is,
   for (unsigned int off = start;off != end + inc; off += inc)
   {
     const struct TBI_Command *pos = &is->commands[off];
+    int cancelled;
 
     if (TBI_OC_ADMIN_ADD_INCOMING != pos->oc)
       continue;
     if ( (NULL != ref) &&
-         (ref->details.admin_add_incoming.serial_id ==
-          pos->details.admin_add_incoming.serial_id) )
+         (ref->details.admin_add_incoming.row_id ==
+          pos->details.admin_add_incoming.row_id) )
     {
       total = 0;
       ok = GNUNET_YES;
@@ -229,6 +257,11 @@ build_history (struct InterpreterState *is,
       continue; /* skip until we find the marker */
     if (total >= cmd->details.history.num_results * inc)
       break; /* hit limit specified by command */
+    cancelled = test_cancelled (is,
+                                off);
+    if ( (GNUNET_YES == cancelled) &&
+         (0 == (cmd->details.history.direction & TALER_BANK_DIRECTION_CANCEL)) )
+      continue;
     if ( ( (0 != (cmd->details.history.direction & TALER_BANK_DIRECTION_CREDIT)) &&
            (cmd->details.history.account_number ==
             pos->details.admin_add_incoming.credit_account_no)) ||
@@ -253,12 +286,13 @@ build_history (struct InterpreterState *is,
   for (unsigned int off = start;off != end + inc; off += inc)
   {
     const struct TBI_Command *pos = &is->commands[off];
+    int cancelled;
 
     if (TBI_OC_ADMIN_ADD_INCOMING != pos->oc)
       continue;
     if ( (NULL != ref) &&
-         (ref->details.admin_add_incoming.serial_id ==
-          pos->details.admin_add_incoming.serial_id) )
+         (ref->details.admin_add_incoming.row_id ==
+          pos->details.admin_add_incoming.row_id) )
     {
       total = 0;
       ok = GNUNET_YES;
@@ -268,7 +302,6 @@ build_history (struct InterpreterState *is,
       continue; /* skip until we find the marker */
     if (total >= cmd->details.history.num_results * inc)
       break; /* hit limit specified by command */
-
     if ( ( (0 != (cmd->details.history.direction & TALER_BANK_DIRECTION_CREDIT)) &&
            (cmd->details.history.account_number ==
             pos->details.admin_add_incoming.credit_account_no)) &&
@@ -280,16 +313,24 @@ build_history (struct InterpreterState *is,
       continue;
     }
 
+    cancelled = test_cancelled (is,
+                                off);
+    if ( (GNUNET_YES == cancelled) &&
+         (0 == (cmd->details.history.direction & TALER_BANK_DIRECTION_CANCEL)) )
+      continue;
+
     if ( (0 != (cmd->details.history.direction & TALER_BANK_DIRECTION_CREDIT)) &&
          (cmd->details.history.account_number ==
           pos->details.admin_add_incoming.credit_account_no))
     {
       h[total].direction = TALER_BANK_DIRECTION_CREDIT;
+      if (GNUNET_YES == cancelled)
+        h[total].direction |= TALER_BANK_DIRECTION_CANCEL;
       h[total].details.account_details
         = json_pack ("{s:s, s:s, s:I}",
                      "type",
                      "test",
-                     "bank_uri",
+                     "bank_url",
                      "http://localhost:8080",
                      "account_number",
                      (json_int_t) pos->details.admin_add_incoming.debit_account_no);
@@ -300,11 +341,13 @@ build_history (struct InterpreterState *is,
             pos->details.admin_add_incoming.debit_account_no))
     {
       h[total].direction = TALER_BANK_DIRECTION_DEBIT;
+      if (GNUNET_YES == cancelled)
+        h[total].direction |= TALER_BANK_DIRECTION_CANCEL;
       h[total].details.account_details
         = json_pack ("{s:s, s:s, s:I}",
                      "type",
                      "test",
-                     "bank_uri",
+                     "bank_url",
                      "http://localhost:8080",
                      "account_number",
                      (json_int_t) pos->details.admin_add_incoming.credit_account_no);
@@ -321,19 +364,12 @@ build_history (struct InterpreterState *is,
                      TALER_string_to_amount (pos->details.admin_add_incoming.amount,
                                              &h[total].details.amount));
       /* h[total].execution_date; // unknown here */
-      h[total].serial_id
-        = pos->details.admin_add_incoming.serial_id;
-      {
-        char *ws;
-
-        ws = GNUNET_STRINGS_data_to_string_alloc (&pos->details.admin_add_incoming.wtid,
-                                                  sizeof (struct TALER_WireTransferIdentifierRawP));
-        GNUNET_asprintf (&h[total].details.wire_transfer_subject,
-                         "%s %s",
-                         ws,
-                         pos->details.admin_add_incoming.exchange_base_url);
-        GNUNET_free (ws);
-      }
+      h[total].row_id
+        = pos->details.admin_add_incoming.row_id;
+      GNUNET_asprintf (&h[total].details.wire_transfer_subject,
+                       "%s %s",
+                       pos->details.admin_add_incoming.subject,
+                       pos->details.admin_add_incoming.exchange_base_url);
       total++;
     }
   }
@@ -371,7 +407,7 @@ print_expected (struct History *h,
                 (unsigned long long) i,
                 (TALER_BANK_DIRECTION_CREDIT == h[i].direction) ? "+" : "-",
                 TALER_amount2s (&h[i].details.amount),
-                (unsigned long long) h[i].serial_id,
+                (unsigned long long) h[i].row_id,
                 h[i].details.wire_transfer_subject,
                 acc);
     if (NULL != acc)
@@ -488,26 +524,42 @@ static void
 interpreter_run (void *cls);
 
 
+/**
+ * Run the next command.
+ *
+ * @param is interpreter to progress
+ */
+static void
+next (struct InterpreterState *is)
+{
+  is->ip++;
+  is->task = GNUNET_SCHEDULER_add_now (&interpreter_run,
+                                       is);
+}
+
+
 /**
  * Function called upon completion of our /admin/add/incoming request.
  *
  * @param cls closure with the interpreter state
  * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful status request
  *                    0 if the bank's reply is bogus (fails to follow the protocol)
- * @param serial_id unique ID of the wire transfer in the bank's records; UINT64_MAX on error
+ * @param ec taler status code
+ * @param row_id unique ID of the wire transfer in the bank's records; UINT64_MAX on error
  * @param json detailed response from the HTTPD, or NULL if reply was not in JSON
  */
 static void
 add_incoming_cb (void *cls,
                  unsigned int http_status,
-                 uint64_t serial_id,
+                 enum TALER_ErrorCode ec,
+                 uint64_t row_id,
                  const json_t *json)
 {
   struct InterpreterState *is = cls;
   struct TBI_Command *cmd = &is->commands[is->ip];
 
   cmd->details.admin_add_incoming.aih = NULL;
-  cmd->details.admin_add_incoming.serial_id = serial_id;
+  cmd->details.admin_add_incoming.row_id = row_id;
   if (cmd->details.admin_add_incoming.expected_response_code != http_status)
   {
     GNUNET_break (0);
@@ -522,9 +574,7 @@ add_incoming_cb (void *cls,
     fail (is);
     return;
   }
-  is->ip++;
-  is->task = GNUNET_SCHEDULER_add_now (&interpreter_run,
-                                       is);
+  next (is);
 }
 
 
@@ -538,16 +588,18 @@ add_incoming_cb (void *cls,
  *                    #MHD_HTTP_NO_CONTENT if there are no more results; on success the
  *                    last callback is always of this status (even if `abs(num_results)` were
  *                    already returned).
+ * @param ec taler status code
  * @param dir direction of the transfer
- * @param serial_id monotonically increasing counter corresponding to the transaction
+ * @param row_id monotonically increasing counter corresponding to the transaction
  * @param details details about the wire transfer
  * @param json detailed response from the HTTPD, or NULL if reply was not in JSON
  */
 static void
 history_cb (void *cls,
             unsigned int http_status,
+            enum TALER_ErrorCode ec,
             enum TALER_BANK_Direction dir,
-            uint64_t serial_id,
+            uint64_t row_id,
             const struct TALER_BANK_TransferDetails *details,
             const json_t *json)
 {
@@ -580,9 +632,7 @@ history_cb (void *cls,
       fail (is);
       return;
     }
-    is->ip++;
-    is->task = GNUNET_SCHEDULER_add_now (&interpreter_run,
-                                         is);
+    next (is);
     return;
   }
   if (GNUNET_OK !=
@@ -612,6 +662,38 @@ history_cb (void *cls,
 }
 
 
+/**
+ * Callbacks of this type are used to serve the result of asking
+ * the bank to reject an incoming wire transfer.
+ *
+ * @param cls closure
+ * @param http_status HTTP response code, #MHD_HTTP_NO_CONTENT (204) for successful status request;
+ *                    #MHD_HTTP_NOT_FOUND if the rowid is unknown;
+ *                    0 if the bank's reply is bogus (fails to follow the protocol),
+ * @param ec detailed error code
+ */
+static void
+reject_cb (void *cls,
+           unsigned int http_status,
+           enum TALER_ErrorCode ec)
+{
+  struct InterpreterState *is = cls;
+  struct TBI_Command *cmd = &is->commands[is->ip];
+
+  cmd->details.reject.rh = NULL;
+  if (MHD_HTTP_NO_CONTENT != http_status)
+  {
+    GNUNET_break (0);
+    fprintf (stderr,
+             "Unexpected response code %u:\n",
+             http_status);
+    fail (is);
+    return;
+  }
+  next (is);
+}
+
+
 /**
  * Run the main interpreter loop that performs bank operations.
  *
@@ -658,15 +740,29 @@ interpreter_run (void *cls)
       fail (is);
       return;
     }
-    GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_NONCE,
-                                &cmd->details.admin_add_incoming.wtid,
-                                sizeof (cmd->details.admin_add_incoming.wtid));
+    GNUNET_break (NULL != cmd->details.admin_add_incoming.subject);
+    switch (cmd->details.admin_add_incoming.debit_account_no)
+    {
+    case 1:
+      auth.details.basic.username = "Bank";
+      break;
+    case 2:
+      auth.details.basic.username = "Exchange";
+      break;
+    case 3:
+      auth.details.basic.username = "user3";
+      auth.details.basic.password = "pass3";
+      break;
+    default:
+      GNUNET_break (0);
+      break;
+    }
     cmd->details.admin_add_incoming.aih
       = TALER_BANK_admin_add_incoming (is->ctx,
                                        "http://localhost:8080",
                                        &auth,
                                        cmd->details.admin_add_incoming.exchange_base_url,
-                                       &cmd->details.admin_add_incoming.wtid,
+                                       cmd->details.admin_add_incoming.subject,
                                        &amount,
                                        cmd->details.admin_add_incoming.debit_account_no,
                                        cmd->details.admin_add_incoming.credit_account_no,
@@ -693,9 +789,21 @@ interpreter_run (void *cls)
       ref = NULL;
     }
     if (NULL != ref)
-      rowid = ref->details.admin_add_incoming.serial_id;
+      rowid = ref->details.admin_add_incoming.row_id;
     else
       rowid = UINT64_MAX;
+    switch (cmd->details.history.account_number)
+    {
+    case 1:
+      auth.details.basic.username = "Bank";
+      break;
+    case 2:
+      auth.details.basic.username = "Exchange";
+      break;
+    default:
+      GNUNET_break (0);
+      break;
+    }
     cmd->details.history.hh
       = TALER_BANK_history (is->ctx,
                             "http://localhost:8080",
@@ -722,7 +830,6 @@ interpreter_run (void *cls)
                                            &amount));
     {
       char *subject;
-      char *expect;
 
       if (GNUNET_OK !=
           TALER_FAKEBANK_check (is->fakebank,
@@ -736,22 +843,17 @@ interpreter_run (void *cls)
         fail (is);
         return;
       }
-      expect = GNUNET_STRINGS_data_to_string_alloc (&ref->details.admin_add_incoming.wtid,
-                                                    sizeof (ref->details.admin_add_incoming.wtid));
-      if (0 != strcmp (subject, expect))
+      if (0 != strcmp (ref->details.admin_add_incoming.subject,
+                       subject))
       {
-        GNUNET_free (expect);
         GNUNET_free (subject);
         GNUNET_break (0);
         fail (is);
         return;
       }
       GNUNET_free (subject);
-      GNUNET_free (expect);
     }
-    is->ip++;
-    is->task = GNUNET_SCHEDULER_add_now (&interpreter_run,
-                                         is);
+    next (is);
    return;
   case TBI_OC_EXPECT_TRANSFERS_EMPTY:
     if (GNUNET_OK != TALER_FAKEBANK_check_empty (is->fakebank))
@@ -760,9 +862,39 @@ interpreter_run (void *cls)
       fail (is);
       return;
     }
-    is->ip++;
-    is->task = GNUNET_SCHEDULER_add_now (&interpreter_run,
-                                         is);
+    next (is);
+    return;
+  case TBI_OC_REJECT:
+    ref = find_command (is,
+                        cmd->details.reject.cmd_ref);
+    GNUNET_assert (NULL != ref);
+    GNUNET_assert (TBI_OC_ADMIN_ADD_INCOMING == ref->oc);
+    switch (ref->details.admin_add_incoming.credit_account_no)
+    {
+    case 1:
+      auth.details.basic.username = "Bank";
+      break;
+    case 2:
+      auth.details.basic.username = "Exchange";
+      break;
+    default:
+      GNUNET_break (0);
+      break;
+    }
+    cmd->details.reject.rh
+      = TALER_BANK_reject (is->ctx,
+                           "http://localhost:8080",
+                           &auth,
+                           ref->details.admin_add_incoming.credit_account_no,
+                           ref->details.admin_add_incoming.row_id,
+                           &reject_cb,
+                           is);
+    if (NULL == cmd->details.reject.rh)
+    {
+      GNUNET_break (0);
+      fail (is);
+      return;
+    }
     return;
   default:
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -802,7 +934,6 @@ do_shutdown (void *cls)
 {
   struct InterpreterState *is = cls;
   struct TBI_Command *cmd;
-  unsigned int i;
 
   if (NULL != is->timeout_task)
   {
@@ -810,7 +941,7 @@ do_shutdown (void *cls)
     is->timeout_task = NULL;
   }
 
-  for (i=0;TBI_OC_END != (cmd = &is->commands[i])->oc;i++)
+  for (unsigned int i=0;TBI_OC_END != (cmd = &is->commands[i])->oc;i++)
   {
     switch (cmd->oc)
     {
@@ -843,6 +974,17 @@ do_shutdown (void *cls)
       break;
     case TBI_OC_EXPECT_TRANSFERS_EMPTY:
       break;
+    case TBI_OC_REJECT:
+      if (NULL != cmd->details.reject.rh)
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                    "Command %u (%s) did not complete\n",
+                    i,
+                    cmd->label);
+        TALER_BANK_reject_cancel (cmd->details.reject.rh);
+        cmd->details.reject.rh = NULL;
+      }
+      break;
     default:
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Unknown instruction %d at %u (%s)\n",
