@@ -33,12 +33,12 @@
  * Which revision of the Taler protocol is implemented
  * by this library?  Used to determine compatibility.
  */
-#define TALER_PROTOCOL_CURRENT 1
+#define TALER_PROTOCOL_CURRENT 2
 
 /**
  * How many revisions back are we compatible to?
  */
-#define TALER_PROTOCOL_AGE 1
+#define TALER_PROTOCOL_AGE 0
 
 
 /**
@@ -575,6 +575,8 @@ decode_keys_json (const json_t *resp_obj,
                                    &pub),
       GNUNET_JSON_spec_absolute_time ("list_issue_date",
                                       &list_issue_date),
+      GNUNET_JSON_spec_relative_time ("reserve_closing_delay",
+                                      &key_data->reserve_closing_delay),
       GNUNET_JSON_spec_end()
     };
 
@@ -865,7 +867,7 @@ keys_completed_cb (void *cls,
       {
         /* offsets will map 1:1 */
         unsigned int off = aold->denom_keys[j] - kd_old.denom_keys;
-
+        /* FIXME(#5315): this should not be an assert! */
         GNUNET_assert (off < kd_old.num_denom_keys);
         anew->denom_keys[j] = &kd.denom_keys[off];
       }
@@ -954,7 +956,7 @@ MAH_handle_is_ready (struct TALER_EXCHANGE_Handle *h)
  *
  * @param h handle for the exchange
  * @param path Taler API path (i.e. "/reserve/withdraw")
- * @return the full URI to use with cURL
+ * @return the full URL to use with cURL
  */
 char *
 MAH_path_to_url (struct TALER_EXCHANGE_Handle *h,
@@ -970,7 +972,7 @@ MAH_path_to_url (struct TALER_EXCHANGE_Handle *h,
  *
  * @param base_url base URL of the exchange (i.e. "http://exchange/")
  * @param path Taler API path (i.e. "/reserve/withdraw")
- * @return the full URI to use with cURL
+ * @return the full URL to use with cURL
  */
 char *
 MAH_path_to_url2 (const char *base_url,
@@ -1125,7 +1127,7 @@ request_keys (struct TALER_EXCHANGE_Handle *exchange)
   GNUNET_assert (NULL == exchange->kr);
   kr = GNUNET_new (struct KeysRequest);
   kr->exchange = exchange;
-  if (GNUNET_YES == MAH_handle_is_ready (exchange))
+  if (GNUNET_YES == MAH_handle_is_ready (exchange) && !TALER_EXCHANGE_API_DISABLE_CHERRYPICKING)
   {
     char *arg;
 
