@@ -96,12 +96,14 @@ main (int argc,
       char *const *argv)
 {
   const struct GNUNET_GETOPT_CommandLineOption options[] = {
-    {'r', "reset", NULL,
-     "reset database (DANGEROUS: all existing data is lost!)", 0,
-     &GNUNET_GETOPT_set_one, &reset_db},
-    {'g', "gc", NULL,
-     "garbage collect database", 0,
-     &GNUNET_GETOPT_set_one, &gc_db},
+    GNUNET_GETOPT_option_flag ('r',
+                                  "reset",
+                                  "reset database (DANGEROUS: all existing data is lost!)",
+                                  &reset_db),
+    GNUNET_GETOPT_option_flag ('g',
+                                  "gc",
+                                  "garbage collect database",
+                                  &gc_db),
     GNUNET_GETOPT_OPTION_END
   };
 
