@@ -123,6 +123,11 @@ struct AggregationUnit
    */
   struct GNUNET_HashCode h_wire;
 
+  /**
+   * Hash code of contract we are currently looking into.
+   */
+  const struct GNUNET_HashCode *h_contract;
+
   /**
    * Wire transfer identifier we use.
    */
@@ -374,6 +379,7 @@ update_fees (struct WirePlugin *wp,
 				     p->start_date,
 				     p->end_date,
 				     &p->wire_fee,
+				     &p->closing_fee,
 				     &p->master_sig);
     if (qs < 0)
     {
@@ -567,6 +573,53 @@ exchange_serve_process_config ()
 }
 
 
+/**
+ * Callback invoked with information about refunds applicable
+ * to a particular coin.  Subtract refunded amount(s) from
+ * the aggregation unit's total amount.
+ *
+ * @param cls closure with a `struct AggregationUnit *`
+ * @param merchant_pub public key of merchant who authorized refund
+ * @param merchant_sig signature of merchant authorizing refund
+ * @param h_contract hash of contract being refunded
+ * @param rtransaction_id refund transaction ID
+ * @param amount_with_fee amount being refunded
+ * @param refund_fee fee the exchange keeps for the refund processing
+ * @return #GNUNET_OK to continue to iterate, #GNUNET_SYSERR to stop
+ */
+static int
+refund_by_coin_cb (void *cls,
+		   const struct TALER_MerchantPublicKeyP *merchant_pub,
+		   const struct TALER_MerchantSignatureP *merchant_sig,
+		   const struct GNUNET_HashCode *h_contract,
+		   uint64_t rtransaction_id,
+		   const struct TALER_Amount *amount_with_fee,
+		   const struct TALER_Amount *refund_fee)
+{
+  struct AggregationUnit *au = cls;
+
+  /* TODO: potential optimization: include these conditions
+     in the SELECT! */
+  if (0 != memcmp (merchant_pub,
+		   &au->merchant_pub,
+		   sizeof (struct TALER_MerchantPublicKeyP)))
+    return GNUNET_OK; /* different merchant */
+  if (0 != memcmp (h_contract,
+		   au->h_contract,
+		   sizeof (struct GNUNET_HashCode)))
+    return GNUNET_OK; /* different contract */
+  if (GNUNET_OK !=
+      TALER_amount_subtract (&au->total_amount,
+			     &au->total_amount,
+			     amount_with_fee))
+  {
+    GNUNET_break (0);
+    return GNUNET_SYSERR;
+  }
+  return GNUNET_OK;
+}
+
+
 /**
  * Function called with details about deposits that have been made,
  * with the goal of executing the corresponding wire transaction.
@@ -609,6 +662,20 @@ deposit_cb (void *cls,
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
   au->row_id = row_id;
+
+  au->h_contract = h_contract_terms;
+  qs = db_plugin->select_refunds_by_coin (db_plugin->cls,
+					  au->session,
+					  coin_pub,
+					  &refund_by_coin_cb,
+					  au);
+  au->h_contract = NULL;
+  if (0 > qs)
+  {
+    GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+    return qs;
+  }
+
   GNUNET_assert (NULL == au->wire);
   au->wire = json_incref ((json_t *) wire);
   if (GNUNET_OK !=
@@ -624,8 +691,10 @@ deposit_cb (void *cls,
                               &au->wtid,
                               sizeof (au->wtid));
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Starting aggregation under H(WTID)=%s\n",
-              TALER_B2S (&au->wtid));
+              "Starting aggregation under H(WTID)=%s, starting amount %s at %llu\n",
+              TALER_B2S (&au->wtid),
+              TALER_amount2s (amount_with_fee),
+              (unsigned long long) row_id);
 
   au->wp = find_plugin (extract_type (au->wire));
   if (NULL == au->wp)
@@ -728,6 +797,20 @@ aggregate_cb (void *cls,
     /* Skip this one, but keep going! */
     return GNUNET_DB_STATUS_SUCCESS_ONE_RESULT;
   }
+
+  au->h_contract = h_contract_terms;
+  qs = db_plugin->select_refunds_by_coin (db_plugin->cls,
+					  au->session,
+					  coin_pub,
+					  &refund_by_coin_cb,
+					  au);
+  au->h_contract = NULL;
+  if (0 > qs)
+  {
+    GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+    return qs;
+  }
+
   if (au->rows_offset >= aggregation_limit)
   {
     /* Bug: we asked for at most #aggregation_limit results! */
@@ -759,8 +842,9 @@ aggregate_cb (void *cls,
     return qs;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-	      "Added row %llu to aggregation\n",
-	      (unsigned long long) row_id);
+	      "Added row %llu with %s to aggregation\n",
+	      (unsigned long long) row_id,
+              TALER_amount2s (&delta));
   return GNUNET_DB_STATUS_SUCCESS_ONE_RESULT;
 }
 
@@ -961,6 +1045,7 @@ expired_reserve_cb (void *cls,
 
   GNUNET_assert (NULL == ctc);
   now = GNUNET_TIME_absolute_get ();
+  (void) GNUNET_TIME_round_abs (&now);
 
   /* lookup wire plugin */
   type = extract_type (account_details);
@@ -1101,6 +1186,7 @@ run_reserve_closures (void *cls)
   enum GNUNET_DB_QueryStatus qs;
   const struct GNUNET_SCHEDULER_TaskContext *tc;
   struct ExpiredReserveContext erc;
+  struct GNUNET_TIME_Absolute now;
 
   task = NULL;
   reserves_idle = GNUNET_NO;
@@ -1117,9 +1203,12 @@ run_reserve_closures (void *cls)
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
+  db_plugin->preflight (db_plugin->cls,
+                        session);
   if (GNUNET_OK !=
       db_plugin->start (db_plugin->cls,
-			session))
+			session,
+                        "aggregator reserve closures"))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to start database transaction!\n");
@@ -1129,9 +1218,11 @@ run_reserve_closures (void *cls)
   }
   erc.session = session;
   erc.async_cont = GNUNET_NO;
+  now = GNUNET_TIME_absolute_get ();
+  (void) GNUNET_TIME_round_abs (&now);
   qs = db_plugin->get_expired_reserves (db_plugin->cls,
 					session,
-					GNUNET_TIME_absolute_get (),
+					now,
 					&expired_reserve_cb,
 					&erc);
   switch (qs)
@@ -1325,7 +1416,8 @@ run_aggregation (void *cls)
        transaction to mark all* of the selected deposits as minor! */
     if (GNUNET_OK !=
         db_plugin->start (db_plugin->cls,
-                          session))
+                          session,
+                          "aggregator mark tiny transactions"))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Failed to start database transaction!\n");
@@ -1679,9 +1771,12 @@ run_transfers (void *cls)
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
+  db_plugin->preflight (db_plugin->cls,
+                        session);
   if (GNUNET_OK !=
       db_plugin->start (db_plugin->cls,
-                        session))
+                        session,
+                        "aggregator run transfer"))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to start database transaction!\n");
