@@ -39,7 +39,7 @@
  * release version, and the format is NOT the same that semantic
  * versioning uses either.
  */
-#define TALER_PROTOCOL_VERSION "1:0:1"
+#define TALER_PROTOCOL_VERSION "2:0:0"
 
 
 /**
@@ -677,7 +677,7 @@ reload_keys_denom_iter (void *cls,
   expire_deposit = GNUNET_TIME_absolute_ntoh (dki->issue.properties.expire_deposit);
   if (expire_deposit.abs_value_us < now.abs_value_us)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Skipping expired denomination key `%s'\n",
                 alias);
     return GNUNET_OK;
@@ -711,6 +711,7 @@ reload_keys_denom_iter (void *cls,
     arc.revocation_master_sig = revocation_master_sig;
     if (GNUNET_OK !=
 	TEH_DB_run_transaction (NULL,
+                                "add denomination key revocations",
 				NULL,
 				&add_revocations_transaction,
 				&arc))
@@ -729,7 +730,7 @@ reload_keys_denom_iter (void *cls,
   start = GNUNET_TIME_absolute_ntoh (dki->issue.properties.start);
   if (start.abs_value_us > horizon.abs_value_us)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Skipping future denomination key `%s' (%s), validity starts at %s\n",
                 alias,
 		GNUNET_h2s (&dki->issue.properties.denom_hash),
@@ -739,6 +740,7 @@ reload_keys_denom_iter (void *cls,
 
   if (GNUNET_OK !=
       TEH_DB_run_transaction (NULL,
+                              "add denomination key",
 			      NULL,
 			      &add_denomination_transaction,
 			      (void *) dki))
@@ -814,7 +816,7 @@ reload_keys_sign_iter (void *cls,
   if (GNUNET_TIME_absolute_ntoh (ski->issue.start).abs_value_us >
       horizon.abs_value_us)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Skipping future signing key `%s'\n",
                 filename);
     return GNUNET_OK;
@@ -823,7 +825,7 @@ reload_keys_sign_iter (void *cls,
   if (GNUNET_TIME_absolute_ntoh (ski->issue.expire).abs_value_us <
       now.abs_value_us)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Skipping expired signing key `%s'\n",
                 filename);
     return GNUNET_OK;
@@ -1183,6 +1185,7 @@ build_keys_response (const struct ResponseFactoryContext *rfc,
   struct TALER_ExchangeKeySetPS ks;
   struct TALER_ExchangeSignatureP sig;
   char *keys_json;
+  struct GNUNET_TIME_Relative reserve_closing_delay;
   void *keys_jsonz;
   size_t keys_jsonz_size;
   int comp;
@@ -1287,13 +1290,29 @@ build_keys_response (const struct ResponseFactoryContext *rfc,
                  GNUNET_CRYPTO_eddsa_sign (&rfc->key_state->current_sign_key_issue.signkey_priv.eddsa_priv,
                                            &ks.purpose,
                                            &sig.eddsa_signature));
-
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_time (cfg,
+                                           "exchangedb",
+                                           "IDLE_RESERVE_EXPIRATION_TIME",
+                                           &reserve_closing_delay))
+  {
+    GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
+                               "exchangedb",
+                               "IDLE_RESERVE_EXPIRATION_TIME");
+    /* use default */
+    reserve_closing_delay = GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_WEEKS,
+                                                           4);
+  }
   /* Build /keys response */
-  keys = json_pack ("{s:s, s:o, s:O, s:O, s:o, s:o, s:o, s:o, s:o}",
+  keys = json_pack ("{s:s, s:o, s:o, s:O, s:O,"
+                    " s:o, s:o, s:o, s:o, s:o}",
+                    /* 1-5 */
                     "version", TALER_PROTOCOL_VERSION,
                     "master_public_key", GNUNET_JSON_from_data_auto (&TEH_master_public_key),
+                    "reserve_closing_delay", GNUNET_JSON_from_time_rel (reserve_closing_delay),
                     "signkeys", rfc->sign_keys_array,
                     "payback", rfc->payback_array,
+                    /* 6-10 */
                     "denoms", rbc.denom_keys_array,
                     "auditors", rbc.auditors_array,
                     "list_issue_date", GNUNET_JSON_from_time_abs (rfc->key_state->reload_time),
@@ -1499,7 +1518,7 @@ make_fresh_key_state ()
 		  GNUNET_h2s (&dke->denom_key_hash),
 		  dke);
   }
-  
+
   /* Determine size of `krd_array` by counting number of discrete
      denomination key starting times. */
   last = GNUNET_TIME_UNIT_ZERO_ABS;
@@ -1662,15 +1681,41 @@ TEH_KS_denomination_key_lookup (const struct TEH_KS_StateHandle *key_state,
 				enum TEH_KS_DenominationKeyUse use)
 {
   struct GNUNET_HashCode hc;
+
+  GNUNET_CRYPTO_rsa_public_key_hash (denom_pub->rsa_public_key,
+                                     &hc);
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Looking for denom: '%s..'\n",
+              TALER_B2S (&hc));
+
+  return TEH_KS_denomination_key_lookup_by_hash (key_state,
+                                                 &hc,
+                                                 use);
+}
+
+
+/**
+ * Look up the issue for a denom public key.  Note that the result
+ * is only valid while the @a key_state is not released!
+ *
+ * @param key_state state to look in
+ * @param denom_pub_hash hash of denomination public key
+ * @param use purpose for which the key is being located
+ * @return the denomination key issue,
+ *         or NULL if denom_pub could not be found (or is not valid at this time for the given @a use)
+ */
+struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *
+TEH_KS_denomination_key_lookup_by_hash (const struct TEH_KS_StateHandle *key_state,
+                                        const struct GNUNET_HashCode *denom_pub_hash,
+                                        enum TEH_KS_DenominationKeyUse use)
+{
   struct TALER_EXCHANGEDB_DenominationKeyIssueInformation *dki;
   struct GNUNET_TIME_Absolute now;
   const struct GNUNET_CONTAINER_MultiHashMap *map;
 
-  GNUNET_CRYPTO_rsa_public_key_hash (denom_pub->rsa_public_key,
-                                     &hc);
   map = (TEH_KS_DKU_PAYBACK == use) ? key_state->revoked_map : key_state->denomkey_map;
   dki = GNUNET_CONTAINER_multihashmap_get (map,
-					   &hc);
+					   denom_pub_hash);
   if (NULL == dki)
     return NULL;
   now = GNUNET_TIME_absolute_get ();
@@ -1679,7 +1724,7 @@ TEH_KS_denomination_key_lookup (const struct TEH_KS_StateHandle *key_state,
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
 		"Not returning DKI for %s, as start time is in the future\n",
-		GNUNET_h2s (&hc));
+		GNUNET_h2s (denom_pub_hash));
     return NULL;
   }
   now = GNUNET_TIME_absolute_get ();
@@ -1691,7 +1736,7 @@ TEH_KS_denomination_key_lookup (const struct TEH_KS_StateHandle *key_state,
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
 		  "Not returning DKI for %s, as time to create coins has passed\n",
-		  GNUNET_h2s (&hc));
+		  GNUNET_h2s (denom_pub_hash));
       return NULL;
     }
     break;
@@ -1701,7 +1746,7 @@ TEH_KS_denomination_key_lookup (const struct TEH_KS_StateHandle *key_state,
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
 		  "Not returning DKI for %s, as time to spend coin has passed\n",
-		  GNUNET_h2s (&hc));
+		  GNUNET_h2s (denom_pub_hash));
       return NULL;
     }
     break;
@@ -1711,7 +1756,7 @@ TEH_KS_denomination_key_lookup (const struct TEH_KS_StateHandle *key_state,
     {
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
 		  "Not returning DKI for %s, as time to payback coin has passed\n",
-		  GNUNET_h2s (&hc));
+		  GNUNET_h2s (denom_pub_hash));
       return NULL;
     }
     break;
