@@ -285,7 +285,6 @@ refund_transaction (void *cls,
   }
 
   /* check if we already send the money for the /deposit */
-  // FIXME: DB API...
   qs = TEH_plugin->test_deposit_done (TEH_plugin->cls,
 				      session,
 				      dep);
@@ -453,6 +452,7 @@ verify_and_execute_refund (struct MHD_Connection *connection,
   }
   if (GNUNET_OK !=
       TEH_DB_run_transaction (connection,
+                              "run refund",
 			      &mhd_ret,
 			      &refund_transaction,
 			      (void *) refund))
