@@ -70,15 +70,15 @@ TEH_RESPONSE_can_compress (struct MHD_Connection *connection)
 				    MHD_HTTP_HEADER_ACCEPT_ENCODING);
   if (NULL == ae)
     return MHD_NO;
+  if (0 == strcmp (ae,
+                   "*"))
+    return MHD_YES;
   de = strstr (ae,
 	       "deflate");
   if (NULL == de)
     return MHD_NO;
-  if (0 == strcmp (de,
-                   "*"))
-    return MHD_YES;
   if ( ( (de == ae) ||
-	 ( de[-1] == ',') ||
+	 (de[-1] == ',') ||
 	 (de[-1] == ' ') ) &&
        ( (de[strlen ("deflate")] == '\0') ||
 	 (de[strlen ("deflate")] == ',') ||
@@ -541,18 +541,18 @@ TEH_RESPONSE_compile_transaction_history (const struct TALER_EXCHANGEDB_Transact
 
         ms.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_MELT);
         ms.purpose.size = htonl (sizeof (struct TALER_RefreshMeltCoinAffirmationPS));
-        ms.session_hash = melt->session_hash;
+        ms.rc = melt->session.rc;
         TALER_amount_hton (&ms.amount_with_fee,
-                           &melt->amount_with_fee);
+                           &melt->session.amount_with_fee);
         TALER_amount_hton (&ms.melt_fee,
                            &melt->melt_fee);
-        ms.coin_pub = melt->coin.coin_pub;
+        ms.coin_pub = melt->session.coin.coin_pub;
 	/* internal sanity check before we hand out a bogus sig... */
         if (GNUNET_OK !=
             GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_WALLET_COIN_MELT,
                                         &ms.purpose,
-                                        &melt->coin_sig.eddsa_signature,
-                                        &melt->coin.coin_pub.eddsa_pub))
+                                        &melt->session.coin_sig.eddsa_signature,
+                                        &melt->session.coin.coin_pub.eddsa_pub))
 	{
 	  GNUNET_break (0);
 	  json_decref (history);
@@ -563,10 +563,10 @@ TEH_RESPONSE_compile_transaction_history (const struct TALER_EXCHANGEDB_Transact
 		       json_array_append_new (history,
 					      json_pack ("{s:s, s:o, s:o, s:o, s:o}",
 							 "type", "MELT",
-							 "amount", TALER_JSON_from_amount (&melt->amount_with_fee),
+							 "amount", TALER_JSON_from_amount (&melt->session.amount_with_fee),
 							 "melt_fee", TALER_JSON_from_amount (&melt->melt_fee),
-							 "session_hash", GNUNET_JSON_from_data_auto (&melt->session_hash),
-							 "coin_sig", GNUNET_JSON_from_data_auto (&melt->coin_sig))));
+							 "rc", GNUNET_JSON_from_data_auto (&melt->session.rc),
+							 "coin_sig", GNUNET_JSON_from_data_auto (&melt->session.coin_sig))));
       }
       break;
     case TALER_EXCHANGEDB_TT_REFUND:
@@ -711,7 +711,9 @@ TEH_RESPONSE_compile_reserve_history (const struct TALER_EXCHANGEDB_ReserveHisto
 
   json_history = json_array ();
   ret = 0;
-  for (const struct TALER_EXCHANGEDB_ReserveHistory *pos = rh; NULL != pos; pos = pos->next)
+  for (const struct TALER_EXCHANGEDB_ReserveHistory *pos = rh;
+       NULL != pos;
+       pos = pos->next)
   {
     switch (pos->type)
     {
@@ -731,8 +733,9 @@ TEH_RESPONSE_compile_reserve_history (const struct TALER_EXCHANGEDB_ReserveHisto
       ret |= 1;
       GNUNET_assert (0 ==
                      json_array_append_new (json_history,
-                                            json_pack ("{s:s, s:O, s:o, s:o}",
+                                            json_pack ("{s:s, s:o, s:O, s:o, s:o}",
                                                        "type", "DEPOSIT",
+                                                       "timestamp", GNUNET_JSON_from_time_abs (pos->details.bank->execution_date),
                                                        "sender_account_details", pos->details.bank->sender_account_details,
                                                        "wire_reference", GNUNET_JSON_from_data (pos->details.bank->wire_reference,
                                                                                                 pos->details.bank->wire_reference_size),
@@ -812,7 +815,7 @@ TEH_RESPONSE_compile_reserve_history (const struct TALER_EXCHANGEDB_ReserveHisto
 	  return NULL;
 	}
 
-	GNUNET_assert (0 ==
+        GNUNET_assert (0 ==
 		       json_array_append_new (json_history,
 					      json_pack ("{s:s, s:o, s:o, s:o, s:o, s:o}",
 							 "type", "PAYBACK",
@@ -847,7 +850,7 @@ TEH_RESPONSE_compile_reserve_history (const struct TALER_EXCHANGEDB_ReserveHisto
 	  }
 	}
 	ret |= 2;
-	rcc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED);
+        rcc.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_RESERVE_CLOSED);
 	rcc.purpose.size = htonl (sizeof (struct TALER_ReserveCloseConfirmationPS));
 	rcc.timestamp = GNUNET_TIME_absolute_hton (pos->details.closing->execution_date);
 	TALER_amount_hton (&rcc.closing_amount,
