@@ -26,6 +26,26 @@
 #include "taler-exchange-httpd.h"
 
 
+/**
+ * Initialize wire subsystem.
+ *
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR if we found no valid
+ *         wire methods
+ */
+int
+TEH_WIRE_init (void);
+
+
+/**
+ * Obtain fee structure for @a wire_plugin_name wire transfers.
+ *
+ * @param wire_plugin_name name of the plugin to load fees for
+ * @return JSON object (to be freed by caller) with fee structure
+ */
+json_t *
+TEH_WIRE_get_fees (const char *wire_plugin_name);
+
+
 /**
  * Handle a "/wire" request.
  *
