@@ -36,6 +36,23 @@
 #define DELAY GNUNET_TIME_UNIT_SECONDS
 
 
+/**
+ * Closure for #reject_cb().
+ */
+struct RejectContext
+{
+  /**
+   * Wire transfer subject that was illformed.
+   */
+  char *wtid_s;
+
+  /**
+   * Database session that encountered the problem.
+   */
+  struct TALER_EXCHANGEDB_Session *session;
+};
+
+
 /**
  * Handle to the plugin.
  */
@@ -63,8 +80,8 @@ static struct TALER_EXCHANGEDB_Plugin *db_plugin;
 static int global_ret;
 
 /**
- * Encoded offset in the wire transfer list that we
- * processed last.
+ * Encoded offset in the wire transfer list from where
+ * to start the next query with the bank.
  */
 static void *last_row_off;
 
@@ -73,17 +90,6 @@ static void *last_row_off;
  */
 static size_t last_row_off_size;
 
-/**
- * Encoded offset in the wire transfer list from where
- * to start the next query with the bank.
- */
-static void *start_off;
-
-/**
- * Number of bytes in #start_off.
- */
-static size_t start_off_size;
-
 /**
  * Which wire plugin are we watching?
  */
@@ -99,6 +105,11 @@ static int delay;
  */
 static int test_mode;
 
+/**
+ * Are we running from scratch and should re-process all transactions?
+ */
+static int reset_mode;
+
 /**
  * Next task to run, if any.
  */
@@ -109,6 +120,11 @@ static struct GNUNET_SCHEDULER_Task *task;
  */
 static struct TALER_WIRE_HistoryHandle *hh;
 
+/**
+ * Active request to reject a wire transfer.
+ */
+static struct TALER_WIRE_RejectHandle *rt;
+
 
 /**
  * We're being aborted with CTRL-C (or SIGTERM). Shut down.
@@ -129,12 +145,22 @@ shutdown_task (void *cls)
 				     hh);
     hh = NULL;
   }
+  if (NULL != rt)
+  {
+    char *wtid_s;
+
+    wtid_s = wire_plugin->reject_transfer_cancel (wire_plugin->cls,
+                                                  rt);
+    rt = NULL;
+    GNUNET_free (wtid_s);
+  }
   TALER_EXCHANGEDB_plugin_unload (db_plugin);
   db_plugin = NULL;
   TALER_WIRE_plugin_unload (wire_plugin);
   wire_plugin = NULL;
-  GNUNET_free_non_null (start_off);
-  start_off = NULL;
+  GNUNET_free_non_null (last_row_off);
+  last_row_off = NULL;
+  last_row_off_size = 0;
 }
 
 
@@ -204,11 +230,49 @@ static void
 find_transfers (void *cls);
 
 
+/**
+ * Function called upon completion of the rejection of a wire transfer.
+ *
+ * @param cls closure with the `struct RejectContext`
+ * @param ec error code for the operation
+ */
+static void
+reject_cb (void *cls,
+           enum TALER_ErrorCode ec)
+{
+  struct RejectContext *rtc = cls;
+  enum GNUNET_DB_QueryStatus qs;
+
+  rt = NULL;
+  if (TALER_EC_NONE != ec)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to wire back transfer `%s': %d\n",
+                rtc->wtid_s,
+                ec);
+    GNUNET_free (rtc->wtid_s);
+    db_plugin->rollback (db_plugin->cls,
+			 rtc->session);
+    GNUNET_free (rtc);
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+  GNUNET_free (rtc->wtid_s);
+  qs = db_plugin->commit (db_plugin->cls,
+                          rtc->session);
+  GNUNET_break (0 <= qs);
+  GNUNET_free (rtc);
+  task = GNUNET_SCHEDULER_add_now (&find_transfers,
+                                   NULL);
+}
+
+
 /**
  * Callbacks of this type are used to serve the result of asking
  * the bank for the transaction history.
  *
  * @param cls closure with the `struct TALER_EXCHANGEDB_Session *`
+ * @param ec taler error code
  * @param dir direction of the transfer
  * @param row_off identification of the position at which we are querying
  * @param row_off_size number of bytes in @a row_off
@@ -217,6 +281,7 @@ find_transfers (void *cls);
  */
 static int
 history_cb (void *cls,
+            enum TALER_ErrorCode ec,
 	    enum TALER_BANK_Direction dir,
 	    const void *row_off,
 	    size_t row_off_size,
@@ -224,24 +289,31 @@ history_cb (void *cls,
 {
   struct TALER_EXCHANGEDB_Session *session = cls;
   enum GNUNET_DB_QueryStatus qs;
+  struct TALER_ReservePublicKeyP reserve_pub;
 
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Got history callback, direction %u!\n",
+              (unsigned int) dir);
   if (TALER_BANK_DIRECTION_NONE == dir)
   {
     hh = NULL;
 
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+    if (TALER_EC_NONE != ec)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "Error fetching history: %u!\n",
+                  (unsigned int) ec);
+    }
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "End of list. Committing progress!\n");
     qs = db_plugin->commit (db_plugin->cls,
 			    session);
-    if (0 <= qs)
-    {
-      GNUNET_free_non_null (start_off);
-      start_off = last_row_off;
-      start_off_size = last_row_off_size;
-    }
+    GNUNET_break (0 <= qs);
     if ( (GNUNET_YES == delay) &&
          (test_mode) )
     {
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "Shutdown due to test mode!\n");
       GNUNET_SCHEDULER_shutdown ();
       return GNUNET_OK;
     }
@@ -254,13 +326,47 @@ history_cb (void *cls,
 				       NULL);
     return GNUNET_OK; /* will be ignored anyway */
   }
+  if (NULL != details->wtid_s)
+  {
+    struct RejectContext *rtc;
+
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Wire transfer over %s has invalid subject `%s', sending it back!\n",
+                TALER_amount2s (&details->amount),
+                details->wtid_s);
+    GNUNET_break (0 != row_off_size);
+    if (last_row_off_size != row_off_size)
+    {
+      GNUNET_free_non_null (last_row_off);
+      last_row_off = GNUNET_malloc (row_off_size);
+      last_row_off_size = row_off_size;
+    }
+    memcpy (last_row_off,
+            row_off,
+            row_off_size);
+    rtc = GNUNET_new (struct RejectContext);
+    rtc->session = session;
+    rtc->wtid_s = GNUNET_strdup (details->wtid_s);
+    rt = wire_plugin->reject_transfer (wire_plugin->cls,
+                                       row_off,
+                                       row_off_size,
+                                       &reject_cb,
+                                       rtc);
+    return GNUNET_SYSERR; /* will continue later... */
+  }
+
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Adding wire transfer over %s with subject `%s'\n",
               TALER_amount2s (&details->amount),
-              TALER_B2S (&details->reserve_pub));
+              TALER_B2S (&details->wtid));
+  /* Wire transfer identifier == reserve public key */
+  GNUNET_assert (sizeof (reserve_pub) == sizeof (details->wtid));
+  memcpy (&reserve_pub,
+          &details->wtid,
+          sizeof (reserve_pub));
   qs = db_plugin->reserves_in_insert (db_plugin->cls,
 				      session,
-				      &details->reserve_pub,
+				      &reserve_pub,
 				      &details->amount,
 				      details->execution_date,
 				      details->account_details,
@@ -269,15 +375,13 @@ history_cb (void *cls,
   if (GNUNET_DB_STATUS_HARD_ERROR == qs)
   {
     GNUNET_break (0);
-    db_plugin->rollback (db_plugin->cls,
-			 session);
     GNUNET_SCHEDULER_shutdown ();
     return GNUNET_SYSERR;
   }
   if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
   {
-    db_plugin->rollback (db_plugin->cls,
-                         session);
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Got DB soft error for reserve_in_insert\n");
     /* try again */
     task = GNUNET_SCHEDULER_add_now (&find_transfers,
 				     NULL);
@@ -288,6 +392,7 @@ history_cb (void *cls,
   {
     GNUNET_free_non_null (last_row_off);
     last_row_off = GNUNET_malloc (row_off_size);
+    last_row_off_size = row_off_size;
   }
   memcpy (last_row_off,
 	  row_off,
@@ -308,7 +413,7 @@ find_transfers (void *cls)
   enum GNUNET_DB_QueryStatus qs;
 
   task = NULL;
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Checking for incoming wire transfers\n");
 
   if (NULL == (session = db_plugin->get_session (db_plugin->cls)))
@@ -319,9 +424,12 @@ find_transfers (void *cls)
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
+  db_plugin->preflight (db_plugin->cls,
+                        session);
   if (GNUNET_OK !=
       db_plugin->start (db_plugin->cls,
-                        session))
+                        session,
+                        "wirewatch check for incoming wire transfers"))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to start database transaction!\n");
@@ -329,32 +437,36 @@ find_transfers (void *cls)
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
-  qs = db_plugin->get_latest_reserve_in_reference (db_plugin->cls,
-						   session,
-						   &start_off,
-						   &start_off_size);
-  if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+  if (! reset_mode)
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Failed to obtain starting point for montoring from database!\n");
-    global_ret = GNUNET_SYSERR;
-    GNUNET_SCHEDULER_shutdown ();
-    return;
-  }
-  if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
-  {
-    /* try again */
-    db_plugin->rollback (db_plugin->cls,
-                         session);
-    task = GNUNET_SCHEDULER_add_now (&find_transfers,
-				     NULL);
-    return;
+    qs = db_plugin->get_latest_reserve_in_reference (db_plugin->cls,
+                                                     session,
+                                                     &last_row_off,
+                                                     &last_row_off_size);
+    if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Failed to obtain starting point for montoring from database!\n");
+      global_ret = GNUNET_SYSERR;
+      GNUNET_SCHEDULER_shutdown ();
+      return;
+    }
+    if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
+    {
+      /* try again */
+      task = GNUNET_SCHEDULER_add_now (&find_transfers,
+                                       NULL);
+      return;
+    }
   }
+  GNUNET_assert ( (NULL == last_row_off) ||
+                  ( (NULL != last_row_off) &&
+                    (0 != last_row_off_size) ) );
   delay = GNUNET_YES;
   hh = wire_plugin->get_history (wire_plugin->cls,
 				 TALER_BANK_DIRECTION_CREDIT,
-				 start_off,
-				 start_off_size,
+				 last_row_off,
+				 last_row_off_size,
 				 1024,
 				 &history_cb,
 				 session);
@@ -421,6 +533,10 @@ main (int argc,
 			       "test",
 			       "run in test mode and exit when idle",
 			       &test_mode),
+    GNUNET_GETOPT_option_flag ('r',
+			       "reset",
+			       "start fresh with all transactions in the history",
+			       &reset_mode),
     GNUNET_GETOPT_OPTION_END
   };
 
