@@ -434,18 +434,21 @@ do_deposit (struct Command *cmd)
   /* Build JSON for wire details */
   deposit.receiver_wire_account = json_pack ("{s:s, s:s, s:I}",
                                              "type", "test",
-                                             "bank_uri", "http://localhost:8082/",
+                                             "bank_url", "http://localhost:8082/",
                                              "account_number", (json_int_t) cmd->details.deposit.merchant_account);
   GNUNET_assert (GNUNET_OK ==
                  TALER_JSON_hash (deposit.receiver_wire_account,
                                   &deposit.h_wire));
   deposit.timestamp = GNUNET_TIME_absolute_get ();
+  GNUNET_TIME_round_abs (&deposit.timestamp);
   deposit.wire_deadline = GNUNET_TIME_relative_to_absolute (cmd->details.deposit.wire_deadline);
+  GNUNET_TIME_round_abs (&deposit.wire_deadline);
 
   /* finally, actually perform the DB operation */
   if ( (GNUNET_OK !=
         plugin->start (plugin->cls,
-                       session)) ||
+                       session,
+                       "aggregator-test-1")) ||
        (GNUNET_OK !=
         plugin->insert_deposit (plugin->cls,
                                 session,
@@ -1148,7 +1151,8 @@ run (void *cls)
 				     &issue.properties.denom_hash);
   if ( (GNUNET_OK !=
         plugin->start (plugin->cls,
-                       session)) ||
+                       session,
+                       "aggregator-test-2")) ||
        (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
         plugin->insert_denomination_info (plugin->cls,
                                           session,
