@@ -74,7 +74,7 @@ main (int argc, char ** argv)
     /* End of initialization */
     /* Reserve initialization */
     PERF_TALER_EXCHANGEDB_INIT_CMD_LOOP ("02 - init reserve loop",
-                                     NB_RESERVE_INIT),
+                                         NB_RESERVE_INIT),
     PERF_TALER_EXCHANGEDB_INIT_CMD_CREATE_RESERVE ("02 - reserve"),
     PERF_TALER_EXCHANGEDB_INIT_CMD_INSERT_RESERVE ("02 - insert",
                                                "02 - reserve"),
@@ -131,14 +131,24 @@ main (int argc, char ** argv)
     PERF_TALER_EXCHANGEDB_INIT_CMD_LOOP ("05 - refresh session init loop",
                                      NB_REFRESH_INIT),
     PERF_TALER_EXCHANGEDB_INIT_CMD_START_TRANSACTION (""),
-    PERF_TALER_EXCHANGEDB_INIT_CMD_CREATE_REFRESH_SESSION ("05 - refresh session"),
+    PERF_TALER_EXCHANGEDB_INIT_CMD_LOAD_ARRAY ("05 - denomination load",
+                                               "05 - refresh session init loop",
+                                               "01 - save denomination"),
+    PERF_TALER_EXCHANGEDB_INIT_CMD_LOAD_ARRAY ("05 - reserve load",
+                                               "05 - refresh session init loop",
+                                               "02 - save reserve"),
+    PERF_TALER_EXCHANGEDB_INIT_CMD_CREATE_WITHDRAW ("05 - withdraw",
+                                                    "05 - denomination load",
+                                                    "05 - reserve load"),
+    PERF_TALER_EXCHANGEDB_INIT_CMD_CREATE_REFRESH_SESSION ("05 - refresh session",
+                                                           "05 - withdraw"),
     PERF_TALER_EXCHANGEDB_INIT_CMD_SAVE_ARRAY ("05 - session array",
-                                           "05 - refresh session init loop",
-                                           "05 - refresh session",
-                                           NB_RESERVE_SAVE),
+                                               "05 - refresh session init loop",
+                                               "05 - refresh session",
+                                               NB_RESERVE_SAVE),
     PERF_TALER_EXCHANGEDB_INIT_CMD_COMMIT_TRANSACTION (""),
     PERF_TALER_EXCHANGEDB_INIT_CMD_END_LOOP ("05 - end",
-                                         "05 - refresh session init loop"),
+                                             "05 - refresh session init loop"),
     /* End of refresh session initialization */
     /* Refresh melt initialization */
     PERF_TALER_EXCHANGEDB_INIT_CMD_LOOP ("06 - refresh melt init loop",
@@ -328,9 +338,23 @@ main (int argc, char ** argv)
                                        NB_DEPOSIT_SAVE),
     PERF_TALER_EXCHANGEDB_INIT_CMD_GET_TIME ("29 - start"),
     PERF_TALER_EXCHANGEDB_INIT_CMD_LOOP ("29 - insert refresh session",
-                                     NB_REFRESH_SAVE),
+                                         NB_REFRESH_SAVE),
     PERF_TALER_EXCHANGEDB_INIT_CMD_START_TRANSACTION (""),
-    PERF_TALER_EXCHANGEDB_INIT_CMD_CREATE_REFRESH_SESSION (""),
+    PERF_TALER_EXCHANGEDB_INIT_CMD_LOAD_ARRAY ("29 - denomination load",
+                                               "29 - insert refresh session",
+                                               "01 - save denomination"),
+    PERF_TALER_EXCHANGEDB_INIT_CMD_LOAD_ARRAY ("29 - reserve load",
+                                               "29 - insert refresh session",
+                                               "02 - save reserve"),
+    PERF_TALER_EXCHANGEDB_INIT_CMD_CREATE_WITHDRAW ("29 - withdraw",
+                                                    "29 - denomination load",
+                                                    "29 - reserve load"),
+    PERF_TALER_EXCHANGEDB_INIT_CMD_CREATE_REFRESH_SESSION ("29 - refresh session",
+                                                           "29 - withdraw"),
+    PERF_TALER_EXCHANGEDB_INIT_CMD_SAVE_ARRAY ("29 - session array",
+                                               "29 - insert refresh session",
+                                               "29 - refresh session",
+                                               NB_RESERVE_SAVE),
     PERF_TALER_EXCHANGEDB_INIT_CMD_COMMIT_TRANSACTION (""),
     PERF_TALER_EXCHANGEDB_INIT_CMD_END_LOOP ("",
                                          "29 - insert refresh session"),
