@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2016 Inria
+  Copyright (C) 2016, 2017 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -58,6 +58,21 @@ enum TALER_ErrorCode
    */
   TALER_EC_EXCHANGE_BAD_CONFIGURATION = 4,
 
+  /**
+   * Internal assertion error.
+   */
+  TALER_EC_INTERNAL_INVARIANT_FAILURE = 5,
+
+  /**
+   * Operation timed out.
+   */
+  TALER_EC_TIMEOUT = 6,
+
+  /**
+   * Exchange failed to allocate memory for building JSON reply.
+   */
+  TALER_EC_JSON_ALLOCATION_FAILURE = 7,
+
   /* ********** generic error codes ************* */
 
   /**
@@ -418,6 +433,7 @@ enum TALER_ErrorCode
    */
   TALER_EC_DEPOSIT_INVALID_TIMESTAMP = 1218,
 
+
   /**
    * The respective coin did not have sufficient residual value
    * for the /refresh/melt operation.  The "history" in this
@@ -458,86 +474,26 @@ enum TALER_ErrorCode
    */
   TALER_EC_REFRESH_MELT_DB_STORE_SESSION_ERROR = 1304,
 
-  /**
-   * The exchange failed to store commit data in the
-   * database.
-   * This response is provided with HTTP status code
-   * MHD_HTTP_INTERNAL_ERROR.
-   */
-  TALER_EC_REFRESH_MELT_DB_STORE_COMMIT_ERROR = 1306,
-
-  /**
-   * The exchange is unaware of the denomination key that was
-   * requested for one of the fresh coins.  This response is provided
-   * with HTTP status code MHD_HTTP_BAD_REQUEST.
-   */
-  TALER_EC_REFRESH_MELT_FRESH_DENOMINATION_KEY_NOT_FOUND = 1308,
-
-  /**
-   * The exchange encountered a numeric overflow totaling up
-   * the cost for the refresh operation.  This response is provided
-   * with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
-   */
-  TALER_EC_REFRESH_MELT_COST_CALCULATION_OVERFLOW = 1309,
-
-  /**
-   * During the transaction phase, the exchange could suddenly
-   * no longer find the denomination key that was
-   * used to sign the melted coin.  This response is provided
-   * with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
-   */
-  TALER_EC_REFRESH_MELT_DB_DENOMINATION_KEY_NOT_FOUND = 1310,
-
   /**
    * The exchange encountered melt fees exceeding the melted
    * coin's contribution.  This response is provided
    * with HTTP status code MHD_HTTP_BAD_REQUEST.
    */
-  TALER_EC_REFRESH_MELT_FEES_EXCEED_CONTRIBUTION = 1311,
-
-  /**
-   * The exchange's cost calculation does not add up to the
-   * melt fees specified in the request.  This response is provided
-   * with HTTP status code MHD_HTTP_BAD_REQUEST.
-   */
-  TALER_EC_REFRESH_MELT_FEES_MISSMATCH = 1312,
+  TALER_EC_REFRESH_MELT_FEES_EXCEED_CONTRIBUTION = 1305,
 
   /**
    * The denomination key signature on the melted coin is invalid.
    * This response is provided with HTTP status code
    * MHD_HTTP_BAD_REQUEST.
    */
-  TALER_EC_REFRESH_MELT_DENOMINATION_SIGNATURE_INVALID = 1313,
-
-  /**
-   * The exchange's cost calculation shows that the melt amount
-   * is below the costs of the transaction.  This response is provided
-   * with HTTP status code MHD_HTTP_BAD_REQUEST.
-   */
-  TALER_EC_REFRESH_MELT_AMOUNT_INSUFFICIENT = 1314,
+  TALER_EC_REFRESH_MELT_DENOMINATION_SIGNATURE_INVALID = 1306,
 
   /**
    * The signature made with the coin to be melted is invalid.
    * This response is provided with HTTP status code
    * MHD_HTTP_BAD_REQUEST.
    */
-  TALER_EC_REFRESH_MELT_COIN_SIGNATURE_INVALID = 1315,
-
-  /**
-   * The size of the cut-and-choose dimension of the
-   * blinded coins request does not match #TALER_CNC_KAPPA.
-   * This response is provided with HTTP status code
-   * MHD_HTTP_BAD_REQUEST.
-   */
-  TALER_EC_REFRESH_MELT_CNC_COIN_ARRAY_SIZE_INVALID = 1316,
-
-  /**
-   * The size of the cut-and-choose dimension of the
-   * transfer keys request does not match #TALER_CNC_KAPPA.
-   * This response is provided with HTTP status code
-   * MHD_HTTP_BAD_REQUEST.
-   */
-  TALER_EC_REFRESH_MELT_CNC_TRANSFER_ARRAY_SIZE_INVALID = 1317,
+  TALER_EC_REFRESH_MELT_COIN_SIGNATURE_INVALID = 1307,
 
   /**
    * The exchange failed to obtain the transaction history of the
@@ -546,7 +502,8 @@ enum TALER_ErrorCode
    * This response is provided with HTTP status code
    * MHD_HTTP_INTERNAL_SERVER_ERROR.
    */
-  TALER_EC_REFRESH_MELT_HISTORY_DB_ERROR_INSUFFICIENT_FUNDS = 1318,
+  TALER_EC_REFRESH_MELT_HISTORY_DB_ERROR_INSUFFICIENT_FUNDS = 1308,
+
 
   /**
    * The provided transfer keys do not match up with the
@@ -556,21 +513,13 @@ enum TALER_ErrorCode
    */
   TALER_EC_REFRESH_REVEAL_COMMITMENT_VIOLATION = 1350,
 
-  /**
-   * Failed to blind the envelope to reconstruct the blinded
-   * coins for revealation checks.
-   * This response is provided with HTTP status code
-   * MHD_HTTP_INTERNAL_ERROR.
-   */
-  TALER_EC_REFRESH_REVEAL_BLINDING_ERROR = 1351,
-
   /**
    * Failed to produce the blinded signatures over the coins
    * to be returned.
    * This response is provided with HTTP status code
    * MHD_HTTP_INTERNAL_ERROR.
    */
-  TALER_EC_REFRESH_REVEAL_SIGNING_ERROR = 1352,
+  TALER_EC_REFRESH_REVEAL_SIGNING_ERROR = 1351,
 
   /**
    * The exchange is unaware of the refresh sessino specified in
@@ -578,7 +527,7 @@ enum TALER_ErrorCode
    * This response is provided with HTTP status code
    * MHD_HTTP_BAD_REQUEST.
    */
-  TALER_EC_REFRESH_REVEAL_SESSION_UNKNOWN = 1353,
+  TALER_EC_REFRESH_REVEAL_SESSION_UNKNOWN = 1352,
 
   /**
    * The exchange failed to retrieve valid session data from the
@@ -586,23 +535,14 @@ enum TALER_ErrorCode
    * This response is provided with HTTP status code
    * MHD_HTTP_INTERNAL_ERROR.
    */
-  TALER_EC_REFRESH_REVEAL_DB_FETCH_SESSION_ERROR = 1354,
+  TALER_EC_REFRESH_REVEAL_DB_FETCH_SESSION_ERROR = 1353,
 
   /**
-   * The exchange failed to retrieve order data from the
-   * database.
-   * This response is provided with HTTP status code
+   * The exchange failed to retrieve previously revealed data from the
+   * database.  This response is provided with HTTP status code
    * MHD_HTTP_INTERNAL_ERROR.
    */
-  TALER_EC_REFRESH_REVEAL_DB_FETCH_ORDER_ERROR = 1355,
-
-  /**
-   * The exchange failed to retrieve transfer keys from the
-   * database.
-   * This response is provided with HTTP status code
-   * MHD_HTTP_INTERNAL_ERROR.
-   */
-  TALER_EC_REFRESH_REVEAL_DB_FETCH_TRANSFER_ERROR = 1356,
+  TALER_EC_REFRESH_REVEAL_DB_FETCH_REVEAL_ERROR = 1354,
 
   /**
    * The exchange failed to retrieve commitment data from the
@@ -610,7 +550,7 @@ enum TALER_ErrorCode
    * This response is provided with HTTP status code
    * MHD_HTTP_INTERNAL_ERROR.
    */
-  TALER_EC_REFRESH_REVEAL_DB_FETCH_COMMIT_ERROR = 1357,
+  TALER_EC_REFRESH_REVEAL_DB_COMMIT_ERROR = 1355,
 
   /**
    * The size of the cut-and-choose dimension of the
@@ -618,7 +558,45 @@ enum TALER_ErrorCode
    * This response is provided with HTTP status code
    * MHD_HTTP_BAD_REQUEST.
    */
-  TALER_EC_REFRESH_REVEAL_CNC_TRANSFER_ARRAY_SIZE_INVALID = 1358,
+  TALER_EC_REFRESH_REVEAL_CNC_TRANSFER_ARRAY_SIZE_INVALID = 1356,
+
+  /**
+   * The number of coins to be created in refresh exceeds the limits
+   * of the exchange.
+   * private transfer keys request does not match #TALER_CNC_KAPPA - 1.
+   * This response is provided with HTTP status code
+   * MHD_HTTP_BAD_REQUEST.
+   */
+  TALER_EC_REFRESH_REVEAL_NEW_DENOMS_ARRAY_SIZE_EXCESSIVE = 1357,
+
+  /**
+   * The number of envelopes given does not match the number
+   * of denomination keys given.
+   * This response is provided with HTTP status code
+   * MHD_HTTP_BAD_REQUEST.
+   */
+  TALER_EC_REFRESH_REVEAL_NEW_DENOMS_ARRAY_SIZE_MISSMATCH = 1358,
+
+  /**
+   * The exchange encountered a numeric overflow totaling up
+   * the cost for the refresh operation.  This response is provided
+   * with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_REFRESH_REVEAL_COST_CALCULATION_OVERFLOW = 1359,
+
+  /**
+   * The exchange's cost calculation shows that the melt amount
+   * is below the costs of the transaction.  This response is provided
+   * with HTTP status code MHD_HTTP_BAD_REQUEST.
+   */
+  TALER_EC_REFRESH_REVEAL_AMOUNT_INSUFFICIENT = 1360,
+
+  /**
+   * The exchange is unaware of the denomination key that was
+   * requested for one of the fresh coins.  This response is provided
+   * with HTTP status code MHD_HTTP_BAD_REQUEST.
+   */
+  TALER_EC_REFRESH_REVEAL_FRESH_DENOMINATION_KEY_NOT_FOUND = 1361,
 
 
   /**
@@ -1096,11 +1074,19 @@ enum TALER_ErrorCode
 
   /**
    * We got different currencies for the wire fee and the maximum wire
-   * fee.  This response is provided with HTTP status code
+   * fee.  This response is provided with HTTP status code of
    * MHD_HTTP_INTERNAL_SERVER_ERROR.
    */
   TALER_EC_PAY_WIRE_FEE_CURRENCY_MISSMATCH = 2125,
 
+  /**
+   * The merchant refuses to abort and refund the payment
+   * operation as the payment succeeded already.
+   * This response is provided with HTTP status code of
+   * MHD_HTTP_FORBIDDEN.
+   */
+  TALER_EC_PAY_ABORT_REFUND_REFUSED_PAYMENT_COMPLETE = 2126,
+
 
   /**
    * Integer overflow with sepcified timestamp argument detected.
@@ -1293,12 +1279,24 @@ enum TALER_ErrorCode
    */
   TALER_EC_PROPOSAL_LOOKUP_DB_ERROR = 2502,
 
-
   /**
    * The proposal being looked up is not found on this merchant.
    */
   TALER_EC_PROPOSAL_LOOKUP_NOT_FOUND = 2503,
 
+  /**
+   * The proposal had no timestamp and the backend failed to obtain the local time.
+   * Likely to be an internal error.
+   */
+  TALER_EC_PROPOSAL_NO_LOCALTIME = 2504,
+
+  /**
+   * The order provided to the backend could not be parsed, some
+   * required fields were missing or ill-formed.
+   * Returned as a bad request.
+   */
+  TALER_EC_PROPOSAL_ORDER_PARSE_ERROR = 2505,
+
   /**
    * The client specified an unknown instance for any of the /refund operations
    */
@@ -1336,6 +1334,168 @@ enum TALER_ErrorCode
    */
    TALER_EC_PAY_DB_STORE_PAYMENTS_ERROR = 2605,
 
+  /**
+   * The backend does not know the instance that was supposed to support
+   * the tip.  Likely to be a configuration error. Returned with an
+   * HTTP status code of "NOT FOUND".
+   */
+  TALER_EC_TIP_AUTHORIZE_INSTANCE_UNKNOWN = 2700,
+
+  /**
+   * The backend knows the instance that was supposed to support the
+   * tip, but it was not configured for tipping (i.e. has no exchange
+   * associated with it).  Likely to be a configuration
+   * error. Returned with an HTTP status code of "NOT FOUND".
+   */
+  TALER_EC_TIP_AUTHORIZE_INSTANCE_DOES_NOT_TIP = 2701,
+
+  /**
+   * The reserve that was used to fund the tips has expired.
+   * Returned with an HTTP status code of "not found".
+   */
+  TALER_EC_TIP_AUTHORIZE_RESERVE_EXPIRED = 2702,
+
+  /**
+   * The reserve that was used to fund the tips was not found in the DB.
+   * Returned with an HTTP status code of "not found".
+   */
+  TALER_EC_TIP_AUTHORIZE_RESERVE_UNKNOWN = 2703,
+
+  /**
+   * The backend knows the instance that was supposed to support the
+   * tip, and it was configured for tipping. However, the funds
+   * remaining are insufficient to cover the tip, and the merchant
+   * should top up the reserve.
+   * Returned with an HTTP status code of "PRECONDITION FAILED".
+   */
+  TALER_EC_TIP_AUTHORIZE_INSUFFICIENT_FUNDS = 2704,
+
+  /**
+   * The backend had trouble accessing the database to persist
+   * information about the tip authorization.
+   * Returned with an HTTP status code of internal error.
+   */
+  TALER_EC_TIP_AUTHORIZE_DB_HARD_ERROR = 2705,
+
+  /**
+   * The backend had trouble accessing the database to persist
+   * information about the tip authorization.
+   * The problem might be fixable by repeating the transaction.
+   */
+  TALER_EC_TIP_AUTHORIZE_DB_SOFT_ERROR = 2706,
+
+  /**
+   * The backend had trouble accessing the database to persist
+   * information about enabling tips.
+   * Returned with an HTTP status code of internal error.
+   */
+  TALER_EC_TIP_ENABLE_DB_TRANSACTION_ERROR = 2750,
+
+  /**
+   * The tip ID is unknown.  This could happen if the tip has
+   * expired.  Returned with an HTTP status code of "not found".
+   */
+  TALER_EC_TIP_PICKUP_TIP_ID_UNKNOWN = 2800,
+
+  /**
+   * The amount requested exceeds the remaining tipping balance for this tip ID.
+   * Returned with an HTTP status code of "Conflict" (as it conflicts with
+   * a previous pickup operation).
+   */
+  TALER_EC_TIP_PICKUP_NO_FUNDS = 2801,
+
+  /**
+   * We encountered a DB error, repeating the request may work.
+   */
+  TALER_EC_TIP_PICKUP_DB_ERROR_SOFT = 2802,
+
+  /**
+   * We encountered a DB error, repeating the request will not help.
+   * This is an internal server error.
+   */
+  TALER_EC_TIP_PICKUP_DB_ERROR_HARD = 2803,
+
+  /**
+   * The same pickup ID was already used for picking up a different
+   * amount. This points to a very strange internal error as the
+   * pickup ID is derived from the denomination key which is tied
+   * to a particular amount. Hence this should also be an internal
+   * server error.
+   */
+  TALER_EC_TIP_PICKUP_AMOUNT_CHANGED = 2804,
+
+  /**
+   * We failed to contact the exchange to obtain the denomination
+   * keys.  Returned with a response code "failed dependency" (424).
+   */
+  TALER_EC_TIP_PICKUP_EXCHANGE_DOWN = 2805,
+
+  /**
+   * We contacted the exchange to obtain any denomination
+   * keys, but got no valid keys.
+   * Returned with a response code "failed dependency" (424).
+   */
+  TALER_EC_TIP_PICKUP_EXCHANGE_LACKED_KEYS = 2806,
+
+  /**
+   * We contacted the exchange to obtain at least one of
+   * the denomination keys specified in the request.
+   * Returned with a response code "not found" (404).
+   */
+  TALER_EC_TIP_PICKUP_EXCHANGE_LACKED_KEY = 2807,
+
+  /**
+   * We encountered an arithmetic issue totaling up the
+   * amount to withdraw. Returned with a response code
+   * of "bad request".
+   */
+  TALER_EC_TIP_PICKUP_EXCHANGE_AMOUNT_OVERFLOW = 2808,
+
+  /**
+   * The number of planchets specified exceeded the
+   * limit. Returned with a response code
+   * of "bad request".
+   */
+  TALER_EC_TIP_PICKUP_EXCHANGE_TOO_MANY_PLANCHETS = 2809,
+
+  /**
+   * The tip id is unknown.  This could happen if the
+   * tip id is wrong or the tip authorization expired.
+   */
+  TALER_EC_TIP_QUERY_TIP_ID_UNKNOWN = 2810,
+
+  /**
+   * The instance for check-payment is unknown, likely
+   * a buggy frontend or misconfigured instances.
+   */
+  TALER_EC_CHECK_PAYMENT_INSTANCE_UNKNOWN = 2910,
+
+  /**
+   * We failed to contract terms from our merchant database.
+   * The response is provided with HTTP status code
+   * MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_CHECK_PAYMENT_DB_FETCH_CONTRACT_TERMS_ERROR = 2911,
+
+  /**
+   * We failed to contract terms from our merchant database.
+   * The response is provided with HTTP status code
+   * MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_CHECK_PAYMENT_DB_FETCH_ORDER_ERROR = 2912,
+
+  /**
+   * The order id we're checking is unknown, likely
+   * the frontend did not create the order first.
+   */
+  TALER_EC_CHECK_PAYMENT_ORDER_ID_UNKNOWN = 2913,
+
+  /**
+   * Failed computing a hash code (likely server out-of-memory).
+   * This response is
+   * provided with HTTP status code MHD_HTTP_INTERNAL_SERVER_ERROR.
+   */
+  TALER_EC_CHECK_PAYMENT_FAILED_COMPUTE_PROPOSAL_HASH = 2014,
 
   /* ********** /test API error codes ************* */
 
@@ -1375,6 +1535,87 @@ enum TALER_ErrorCode
    */
   TALER_EC_TEST_RSA_SIGN_ERROR = 4005,
 
+  /* *************** Taler BANK/FAKEBANK error codes *************** */
+
+  /**
+   * The request cannot be served because the client failed to
+   * login.  To be returned along HTTP 401 Unauthorized.
+   */
+  TALER_EC_BANK_REJECT_LOGIN_FAILED = 5312,
+
+  /**
+   * The transaction cannot be rejected becasue it does not exist
+   * at the bank.  To be returned along HTTP 404 Not Found.
+   */
+  TALER_EC_BANK_REJECT_TRANSACTION_NOT_FOUND = 5301,
+
+  /**
+   * The client does not own the account credited by the transaction
+   * which is to be rejected, so it has no rights do reject it.  To be
+   * returned along HTTP 403 Forbidden.
+   */
+  TALER_EC_BANK_REJECT_NO_RIGHTS = 5313,
+
+  /**
+   * The POSTed JSON at /reject was invalid.  To be returned along
+   * HTTP 400 Bad Request.
+   */
+  TALER_EC_BANK_REJECT_JSON_INVALID = 5306,
+
+  /**
+   * A URL parameter for /history was missing.  To be returned along
+   * HTTP 400 Bad Request.
+   */
+  TALER_EC_BANK_HISTORY_PARAMETER_MISSING = 5208,
+
+  /**
+   * A URL parameter for /history was malformed.  To be returned along
+   * HTTP 400 Bad Request.
+   */
+  TALER_EC_BANK_HISTORY_PARAMETER_MALFORMED = 5209,
+
+  /**
+   * The client failed to login for /history.  To be returned along
+   * HTTP 401 Unauthorized.
+   */
+  TALER_EC_BANK_HISTORY_LOGIN_FAILED = 5212,
+
+  /**
+   * The bank had trouble obtaining a valid HTTP response.  To be returned
+   * along status code 0.
+   */
+  TALER_EC_BANK_HISTORY_HTTP_FAILURE = 5213,
+
+  /**
+   * The debit account for /admin/add/incoming is not known to the
+   * bank.  To be returned along HTTP 404 Not Found.
+   */
+  TALER_EC_BANK_ADD_INCOMING_UNKNOWN_ACCOUNT = 5100,
+
+  /**
+   * The client specified the same bank account for both the credit
+   * and the debit account.  The bank will not accomplish this operation.
+   * To be returned along HTTP 403 Forbidden.
+   */
+  TALER_EC_BANK_ADD_INCOMING_SAME_ACCOUNT = 5102,
+
+  /**
+   * The operation would put the client in a debit situation which is
+   * forbidden to them.  To return along HTTP 403 Forbidden.
+   */
+  TALER_EC_BANK_ADD_INCOMING_UNALLOWED_DEBIT = 5103,
+
+  /**
+   * The client POSTed an invalid JSON.  To be returned along HTTP
+   * 400 Bad Request.
+   */
+  TALER_EC_BANK_ADD_INCOMING_JSON_INVALID = 5106,
+
+  /**
+   * The client failed to login for /admin/add/incoming.  To be returned
+   * along HTTP 401 Unauthorized.
+   */
+  TALER_EC_BANK_ADD_INCOMING_LOGIN_FAILED = 5112,
 
   /**
    * End of error code range.
