@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2017 GNUnet e.V.
+  Copyright (C) 2014-2017 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -31,6 +31,13 @@
 /* *********************  /keys *********************** */
 
 
+/**
+ * Temporarily disable cherrypicking until it's fixed.
+ * See #5315.
+ */
+#define TALER_EXCHANGE_API_DISABLE_CHERRYPICKING 1
+
+
 /**
  * List of possible options to be passed to
  * #TALER_EXCHANGE_connect().
@@ -205,6 +212,13 @@ struct TALER_EXCHANGE_Keys
    */
   char *version;
 
+  /**
+   * How long after a reserve went idle will the exchange close it?
+   * This is an approximate number, not cryptographically signed by
+   * the exchange (advisory-only, may change anytime).
+   */
+  struct GNUNET_TIME_Relative reserve_closing_delay;
+
   /**
    * Timestamp indicating the /keys generation.
    */
@@ -716,6 +730,50 @@ TALER_EXCHANGE_refund (struct TALER_EXCHANGE_Handle *exchange,
 		       void *cb_cls);
 
 
+/**
+ * Submit a refund request to the exchange and get the exchange's
+ * response.  This API is used by a merchant.  Note that
+ * while we return the response verbatim to the caller for further
+ * processing, we do already verify that the response is well-formed
+ * (i.e. that signatures included in the response are all valid).  If
+ * the exchange's reply is not well-formed, we return an HTTP status code
+ * of zero to @a cb.
+ *
+ * The @a exchange must be ready to operate (i.e.  have
+ * finished processing the /keys reply).  If this check fails, we do
+ * NOT initiate the transaction with the exchange and instead return NULL.
+ *
+ * @param exchange the exchange handle; the exchange must be ready to operate
+ * @param amount the amount to be refunded; must be larger than the refund fee
+ *        (as that fee is still being subtracted), and smaller than the amount
+ *        (with deposit fee) of the original deposit contribution of this coin
+ * @param refund_fee fee applicable to this coin for the refund
+ * @param h_contract_terms hash of the contact of the merchant with the customer that is being refunded
+ * @param coin_pub coin’s public key of the coin from the original deposit operation
+ * @param rtransaction_id transaction id for the transaction between merchant and customer (of refunding operation);
+ *                        this is needed as we may first do a partial refund and later a full refund.  If both
+ *                        refunds are also over the same amount, we need the @a rtransaction_id to make the disjoint
+ *                        refund requests different (as requests are idempotent and otherwise the 2nd refund might not work).
+ * @param merchant_pub public key of the merchant
+ * @param merchant_sig signature affirming the refund from the merchant
+ * @param cb the callback to call when a reply for this request is available
+ * @param cb_cls closure for the above callback
+ * @return a handle for this request; NULL if the inputs are invalid (i.e.
+ *         signatures fail to verify).  In this case, the callback is not called.
+ */
+struct TALER_EXCHANGE_RefundHandle *
+TALER_EXCHANGE_refund2 (struct TALER_EXCHANGE_Handle *exchange,
+			const struct TALER_Amount *amount,
+			const struct TALER_Amount *refund_fee,
+			const struct GNUNET_HashCode *h_contract_terms,
+			const struct TALER_CoinSpendPublicKeyP *coin_pub,
+			uint64_t rtransaction_id,
+			const struct TALER_MerchantPublicKeyP *merchant_pub,
+			const struct TALER_MerchantSignatureP *merchant_sig,
+			TALER_EXCHANGE_RefundResultCallback cb,
+			void *cb_cls);
+
+
 /**
  * Cancel a refund permission request.  This function cannot be used
  * on a request handle if a response is already served for it.  If
@@ -801,6 +859,11 @@ struct TALER_EXCHANGE_ReserveHistory
        */
       void *wire_reference;
 
+      /**
+       * When did the wire transfer happen?
+       */
+      struct GNUNET_TIME_Absolute timestamp;
+
       /**
        * Number of bytes stored in @e wire_reference.
        */
@@ -966,19 +1029,19 @@ typedef void
 
 
 /**
- * Withdraw a coin from the exchange using a /reserve/withdraw request.  This
- * API is typically used by a wallet.  Note that to ensure that no
- * money is lost in case of hardware failures, the caller must have
- * committed (most of) the arguments to disk before calling, and be
- * ready to repeat the request with the same arguments in case of
- * failures.
+ * Withdraw a coin from the exchange using a /reserve/withdraw
+ * request.  This API is typically used by a wallet to withdraw from a
+ * reserve.
+ *
+ * Note that to ensure that no money is lost in case of hardware
+ * failures, the caller must have committed (most of) the arguments to
+ * disk before calling, and be ready to repeat the request with the
+ * same arguments in case of failures.
  *
  * @param exchange the exchange handle; the exchange must be ready to operate
  * @param pk kind of coin to create
  * @param reserve_priv private key of the reserve to withdraw from
- * @param coin_priv where to fetch the coin's private key,
- *        caller must have committed this value to disk before the call (with @a pk)
- * @param blinding_key where to fetch the coin's blinding key
+ * @param ps secrets of the planchet
  *        caller must have committed this value to disk before the call (with @a pk)
  * @param res_cb the callback to call when the final result for this request is available
  * @param res_cb_cls closure for @a res_cb
@@ -990,12 +1053,43 @@ struct TALER_EXCHANGE_ReserveWithdrawHandle *
 TALER_EXCHANGE_reserve_withdraw (struct TALER_EXCHANGE_Handle *exchange,
                                  const struct TALER_EXCHANGE_DenomPublicKey *pk,
                                  const struct TALER_ReservePrivateKeyP *reserve_priv,
-                                 const struct TALER_CoinSpendPrivateKeyP *coin_priv,
-                                 const struct TALER_DenominationBlindingKeyP *blinding_key,
+                                 const struct TALER_PlanchetSecretsP *ps,
                                  TALER_EXCHANGE_ReserveWithdrawResultCallback res_cb,
                                  void *res_cb_cls);
 
 
+/**
+ * Withdraw a coin from the exchange using a /reserve/withdraw
+ * request.  This API is typically used by a wallet to withdraw a tip
+ * where the reserve's signature was created by the merchant already.
+ *
+ * Note that to ensure that no money is lost in case of hardware
+ * failures, the caller must have committed (most of) the arguments to
+ * disk before calling, and be ready to repeat the request with the
+ * same arguments in case of failures.
+ *
+ * @param exchange the exchange handle; the exchange must be ready to operate
+ * @param pk kind of coin to create
+ * @param reserve_sig signature from the reserve authorizing the withdrawal
+ * @param reserve_pub public key of the reserve to withdraw from
+ * @param ps secrets of the planchet
+ *        caller must have committed this value to disk before the call (with @a pk)
+ * @param res_cb the callback to call when the final result for this request is available
+ * @param res_cb_cls closure for @a res_cb
+ * @return NULL
+ *         if the inputs are invalid (i.e. denomination key not with this exchange).
+ *         In this case, the callback is not called.
+ */
+struct TALER_EXCHANGE_ReserveWithdrawHandle *
+TALER_EXCHANGE_reserve_withdraw2 (struct TALER_EXCHANGE_Handle *exchange,
+                                  const struct TALER_EXCHANGE_DenomPublicKey *pk,
+                                  const struct TALER_ReserveSignatureP *reserve_sig,
+                                  const struct TALER_ReservePublicKeyP *reserve_pub,
+                                  const struct TALER_PlanchetSecretsP *ps,
+                                  TALER_EXCHANGE_ReserveWithdrawResultCallback res_cb,
+                                  void *res_cb_cls);
+
+
 /**
  * Cancel a withdraw status request.  This function cannot be used
  * on a request handle if a response is already served for it.
@@ -1079,7 +1173,7 @@ struct TALER_EXCHANGE_RefreshMeltHandle;
  *                    0 if the exchange's reply is bogus (fails to follow the protocol)
  * @param ec taler-specific error code, #TALER_EC_NONE on success
  * @param noreveal_index choice by the exchange in the cut-and-choose protocol,
- *                    UINT16_MAX on error
+ *                    UINT32_MAX on error
  * @param sign_key exchange key used to sign @a full_response, or NULL
  * @param full_response full response from the exchange (for logging, in case of errors)
  */
@@ -1087,7 +1181,7 @@ typedef void
 (*TALER_EXCHANGE_RefreshMeltCallback) (void *cls,
                                        unsigned int http_status,
 				       enum TALER_ErrorCode ec,
-				       uint16_t noreveal_index,
+				       uint32_t noreveal_index,
                                        const struct TALER_ExchangePublicKeyP *sign_key,
                                        const json_t *full_response);
 
@@ -1192,7 +1286,7 @@ struct TALER_EXCHANGE_RefreshRevealHandle *
 TALER_EXCHANGE_refresh_reveal (struct TALER_EXCHANGE_Handle *exchange,
                                size_t refresh_data_length,
                                const char *refresh_data,
-                               uint16_t noreveal_index,
+                               uint32_t noreveal_index,
                                TALER_EXCHANGE_RefreshRevealCallback reveal_cb,
                                void *reveal_cb_cls);
 
@@ -1275,75 +1369,6 @@ void
 TALER_EXCHANGE_refresh_link_cancel (struct TALER_EXCHANGE_RefreshLinkHandle *rlh);
 
 
-/* ********************* /admin/add/incoming *********************** */
-
-
-/**
- * @brief A /admin/add/incoming Handle
- */
-struct TALER_EXCHANGE_AdminAddIncomingHandle;
-
-
-/**
- * Callbacks of this type are used to serve the result of submitting
- * information about an incoming transaction to a exchange.
- *
- * @param cls closure
- * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful status request
- *                    0 if the exchange's reply is bogus (fails to follow the protocol)
- * @param ec taler-specific error code, #TALER_EC_NONE on success
- * @param full_response full response from the exchange (for logging, in case of errors)
- */
-typedef void
-(*TALER_EXCHANGE_AdminAddIncomingResultCallback) (void *cls,
-                                                  unsigned int http_status,
-						  enum TALER_ErrorCode ec,
-                                                  const json_t *full_response);
-
-
-/**
- * Notify the exchange that we have received an incoming transaction
- * which fills a reserve.  Note that this API is an administrative
- * API and thus not accessible to typical exchange clients, but only
- * to the operators of the exchange.
- *
- * @param exchange the exchange handle; the exchange must be ready to operate
- * @param admin_url URL of the administrative interface of the exchange
- * @param reserve_pub public key of the reserve
- * @param amount amount that was deposited
- * @param execution_date when did we receive the amount
- * @param sender_account_details account information of the sender of the money;
- *        the receiver is always the exchange.
- * @param transfer_details details that uniquely identify the transfer;
- *        used to check for duplicate operations by the exchange
- * @param res_cb the callback to call when the final result for this request is available
- * @param res_cb_cls closure for the above callback
- * @return NULL
- *         if the inputs are invalid (i.e. invalid amount).
- *         In this case, the callback is not called.
- */
-struct TALER_EXCHANGE_AdminAddIncomingHandle *
-TALER_EXCHANGE_admin_add_incoming (struct TALER_EXCHANGE_Handle *exchange,
-                                   const char *admin_url,
-                                   const struct TALER_ReservePublicKeyP *reserve_pub,
-                                   const struct TALER_Amount *amount,
-                                   struct GNUNET_TIME_Absolute execution_date,
-                                   const json_t *sender_account_details,
-                                   const json_t *transfer_details,
-                                   TALER_EXCHANGE_AdminAddIncomingResultCallback res_cb,
-                                   void *res_cb_cls);
-
-
-/**
- * Cancel an add incoming.  This function cannot be used on a request
- * handle if a response is already served for it.
- *
- * @param aai the admin add incoming request handle
- */
-void
-TALER_EXCHANGE_admin_add_incoming_cancel (struct TALER_EXCHANGE_AdminAddIncomingHandle *aai);
-
-
 /* ********************* /track/transfer *********************** */
 
 /**
@@ -1540,8 +1565,7 @@ typedef void
  * @param exchange the exchange handle; the exchange must be ready to operate
  * @param pk kind of coin to pay back
  * @param denom_sig signature over the coin by the exchange using @a pk
- * @param coin_priv the coin's private key,
- * @param blinding_key where to fetch the coin's blinding key
+ * @param ps secret internals of the original planchet
  * @param payback_cb the callback to call when the final result for this request is available
  * @param payback_cb_cls closure for @a payback_cb
  * @return NULL
@@ -1552,8 +1576,7 @@ struct TALER_EXCHANGE_PaybackHandle *
 TALER_EXCHANGE_payback (struct TALER_EXCHANGE_Handle *exchange,
                         const struct TALER_EXCHANGE_DenomPublicKey *pk,
                         const struct TALER_DenominationSignature *denom_sig,
-                        const struct TALER_CoinSpendPrivateKeyP *coin_priv,
-                        const struct TALER_DenominationBlindingKeyP *blinding_key,
+                        const struct TALER_PlanchetSecretsP *ps,
                         TALER_EXCHANGE_PaybackResultCallback payback_cb,
                         void *payback_cb_cls);
 
