@@ -178,6 +178,12 @@
  */
 #define TALER_SIGNATURE_MERCHANT_REFUND_OK 1105
 
+/**
+ * Signature where the merchant confirms that the user replayed
+ * a payment for a browser session.
+ */
+#define TALER_SIGNATURE_MERCHANT_PAY_SESSION 1106
+
 
 /*********************/
 /* Wallet signatures */
@@ -532,9 +538,9 @@ struct TALER_RefreshMeltCoinAffirmationPS
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
   /**
-   * Which melting session should the coin become a part of.
+   * Which melt commitment is made by the wallet.
    */
-  struct GNUNET_HashCode session_hash GNUNET_PACKED;
+  struct TALER_RefreshCommitmentP rc GNUNET_PACKED;
 
   /**
    * How much of the value of the coin should be melted?  This amount
@@ -581,20 +587,16 @@ struct TALER_RefreshMeltConfirmationPS
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
   /**
-   * Hash of the refresh session.
+   * Commitment made in the /refresh/melt.
    */
-  struct GNUNET_HashCode session_hash GNUNET_PACKED;
+  struct TALER_RefreshCommitmentP rc GNUNET_PACKED;
 
   /**
    * Index that the client will not have to reveal, in NBO.
    * Must be smaller than #TALER_CNC_KAPPA.
    */
-  uint16_t noreveal_index GNUNET_PACKED;
+  uint32_t noreveal_index GNUNET_PACKED;
 
-  /**
-   * Zero.
-   */
-  uint16_t reserved GNUNET_PACKED;
 };
 
 
@@ -1283,17 +1285,43 @@ struct TALER_ReserveCloseConfirmationPS
 struct TALER_MerchantRefundConfirmationPS
 {
   /**
-   * Set TALER_SIGNATURE_REFUND_OK.
+   * Set to #TALER_SIGNATURE_MERCHANT_REFUND_OK.
    */
   struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
 
   /**
    * Hashed order id; in case frontend wants to check it.
+   * Hashed without the 0-termination.
    */
   struct GNUNET_HashCode h_order_id GNUNET_PACKED;
 
 };
 
+/**
+ * Used by the merchant to confirm to the frontend that
+ * the user did a payment replay with the current browser session.
+ */
+struct TALER_MerchantPaySessionSigPS
+{
+  /**
+   * Set to #TALER_SIGNATURE_MERCHANT_PAY_SESSION.
+   */
+  struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
+
+  /**
+   * Hashed order id.
+   * Hashed without the 0-termination.
+   */
+  struct GNUNET_HashCode h_order_id GNUNET_PACKED;
+
+  /**
+   * Hashed session id.
+   * Hashed without the 0-termination.
+   */
+  struct GNUNET_HashCode h_session_id GNUNET_PACKED;
+
+};
+
 GNUNET_NETWORK_STRUCT_END
 
 #endif
