@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2016 GNUnet e.V. & Inria
+  Copyright (C) 2017 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -41,9 +41,9 @@ struct TestClosure
   char *currency;
 
   /**
-   * URI of our bank.
+   * URL of our bank.
    */
-  char *bank_uri;
+  char *bank_url;
 
   /**
    * Authentication information.
@@ -187,12 +187,12 @@ test_amount_round (void *cls,
  * Compute purpose for signing.
  *
  * @param account number of the account
- * @param bank_uri URI of the bank
+ * @param bank_url URL of the bank
  * @param[out] wsd purpose to be signed
  */
 static void
 compute_purpose (uint64_t account,
-                 const char *bank_uri,
+                 const char *bank_url,
                  struct TALER_MasterWireDetailsPS *wsd)
 {
   struct GNUNET_HashContext *hc;
@@ -208,8 +208,8 @@ compute_purpose (uint64_t account,
 				   &n,
                                    sizeof (n));
   GNUNET_CRYPTO_hash_context_read (hc,
-				   bank_uri,
-				   strlen (bank_uri) + 1);
+				   bank_url,
+				   strlen (bank_url) + 1);
   GNUNET_CRYPTO_hash_context_finish (hc,
 				     &wsd->h_sepa_details);
 }
@@ -236,7 +236,7 @@ test_wire_validate (void *cls,
   struct TestClosure *tc = cls;
   json_error_t error;
   json_int_t account_no;
-  const char *bank_uri;
+  const char *bank_url;
   const char *sig_s;
   struct TALER_MasterWireDetailsPS wsd;
   struct TALER_MasterSignatureP sig;
@@ -248,7 +248,7 @@ test_wire_validate (void *cls,
 		      0,
 		      "{s:I, s:s}",
 		      "account_number", &account_no,
-                      "bank_uri", &bank_uri))
+                      "bank_url", &bank_url))
   {
     char *dump;
 
@@ -270,14 +270,14 @@ test_wire_validate (void *cls,
                      account_no);
     return TALER_EC_DEPOSIT_INVALID_WIRE_FORMAT_ACCOUNT_NUMBER;
   }
-  if ( (NULL != tc->bank_uri) &&
-       (0 != strcmp (bank_uri,
-                     tc->bank_uri)) )
+  if ( (NULL != tc->bank_url) &&
+       (0 != strcmp (bank_url,
+                     tc->bank_url)) )
   {
     GNUNET_asprintf (emsg,
-                     "Wire specifies bank URI `%s', but this exchange only supports `%s'\n",
-                     bank_uri,
-                     tc->bank_uri);
+                     "Wire specifies bank URL `%s', but this exchange only supports `%s'\n",
+                     bank_url,
+                     tc->bank_url);
     return TALER_EC_DEPOSIT_INVALID_WIRE_FORMAT_BANK;
   }
   if (NULL == master_pub)
@@ -298,7 +298,7 @@ test_wire_validate (void *cls,
     return TALER_EC_DEPOSIT_INVALID_WIRE_FORMAT_SIGNATURE;
   }
   compute_purpose (account_no,
-                   bank_uri,
+                   bank_url,
                    &wsd);
   if (GNUNET_OK !=
       GNUNET_STRINGS_string_to_data (sig_s,
@@ -553,12 +553,14 @@ test_prepare_wire_transfer (void *cls,
  * @param cls closure with the `struct TALER_WIRE_ExecuteHandle`
  * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful status request
  *                    0 if the bank's reply is bogus (fails to follow the protocol)
+ * @param ec error code from the bank
  * @param serial_id unique ID of the wire transfer in the bank's records; UINT64_MAX on error
  * @param json detailed response from the HTTPD, or NULL if reply was not JSON
  */
 static void
 execute_cb (void *cls,
             unsigned int http_status,
+            enum TALER_ErrorCode ec,
             uint64_t serial_id,
             const json_t *json)
 {
@@ -578,13 +580,15 @@ execute_cb (void *cls,
   }
   if (NULL != emsg)
     GNUNET_asprintf (&s,
-                     "%u (%s)",
+                     "%u/%u (%s)",
                      http_status,
+                     (unsigned int) ec,
                      emsg);
   else
     GNUNET_asprintf (&s,
-                     "%u",
-                     http_status);
+                     "%u/%u",
+                     http_status,
+                     (unsigned int) ec);
   eh->cc (eh->cc_cls,
           (MHD_HTTP_OK == http_status) ? GNUNET_OK : GNUNET_SYSERR,
           serial_id,
@@ -612,7 +616,7 @@ test_sign_wire_details (void *cls,
                         struct TALER_MasterSignatureP *sig)
 {
   struct TALER_MasterWireDetailsPS wsd;
-  const char *bank_uri;
+  const char *bank_url;
   const char *type;
   json_int_t account;
   json_error_t err;
@@ -623,7 +627,7 @@ test_sign_wire_details (void *cls,
                       0 /* flags */,
                       "{s:s, s:s, s:I}",
                       "type", &type,
-                      "bank_uri", &bank_uri,
+                      "bank_url", &bank_url,
                       "account_number", &account))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
@@ -640,7 +644,7 @@ test_sign_wire_details (void *cls,
     return GNUNET_SYSERR;
   }
   compute_purpose (account,
-                   bank_uri,
+                   bank_url,
                    &wsd);
   GNUNET_CRYPTO_eddsa_sign (&key->eddsa_priv,
 			    &wsd.purpose,
@@ -676,6 +680,7 @@ test_execute_wire_transfer (void *cls,
   char *emsg;
   const char *json_s;
   const char *exchange_base_url;
+  char *wire_s;
 
   if (NULL == tc->ctx)
   {
@@ -728,16 +733,19 @@ test_execute_wire_transfer (void *cls,
   eh = GNUNET_new (struct TALER_WIRE_ExecuteHandle);
   eh->cc = cc;
   eh->cc_cls = cc_cls;
+  wire_s = GNUNET_STRINGS_data_to_string_alloc (&bf.wtid,
+                                                sizeof (bf.wtid));
   eh->aaih = TALER_BANK_admin_add_incoming (tc->ctx,
-                                            tc->bank_uri,
+                                            tc->bank_url,
                                             &tc->auth,
                                             exchange_base_url,
-                                            &bf.wtid,
+                                            wire_s,
                                             &amount,
                                             (uint64_t) tc->exchange_account_no,
 					    (uint64_t) account_no,
                                             &execute_cb,
                                             eh);
+  GNUNET_free (wire_s);
   json_decref (wire);
   if (NULL == eh->aaih)
   {
@@ -803,6 +811,7 @@ struct TALER_WIRE_HistoryHandle
  *                    #MHD_HTTP_NO_CONTENT if there are no more results; on success the
  *                    last callback is always of this status (even if `abs(num_results)` were
  *                    already returned).
+ * @param ec taler error code
  * @param dir direction of the transfer
  * @param serial_id monotonically increasing counter corresponding to the transaction
  * @param details details about the wire transfer
@@ -811,6 +820,7 @@ struct TALER_WIRE_HistoryHandle
 static void
 bhist_cb (void *cls,
           unsigned int http_status,
+          enum TALER_ErrorCode ec,
           enum TALER_BANK_Direction dir,
           uint64_t serial_id,
           const struct TALER_BANK_TransferDetails *details,
@@ -820,60 +830,86 @@ bhist_cb (void *cls,
   uint64_t bserial_id = GNUNET_htonll (serial_id);
   struct TALER_WIRE_TransferDetails wd;
 
-  if (MHD_HTTP_OK == http_status)
-  {
-    char *subject;
-    char *space;
+  switch (http_status) {
+  case MHD_HTTP_OK:
+    {
+      char *subject;
+      char *space;
 
-    wd.amount = details->amount;
-    wd.execution_date = details->execution_date;
-    subject = GNUNET_strdup (details->wire_transfer_subject);
-    space = strchr (subject, (int) ' ');
-    if (NULL != space)
-    {
-      /* Space separates the actual wire transfer subject from the
-         exchange base URL (if present, expected only for outgoing
-         transactions).  So we cut the string off at the space. */
-      *space = '\0';
-    }
-    /* NOTE: For a real bank, the subject should include a checksum! */
-    if (GNUNET_OK !=
-        GNUNET_STRINGS_string_to_data (subject,
-                                       strlen (subject),
-                                       &wd.reserve_pub,
-                                       sizeof (wd.reserve_pub)))
-    {
-      GNUNET_break (0);
+      wd.amount = details->amount;
+      wd.execution_date = details->execution_date;
+      subject = GNUNET_strdup (details->wire_transfer_subject);
+      space = strchr (subject, (int) ' ');
+      if (NULL != space)
+      {
+        /* Space separates the actual wire transfer subject from the
+           exchange base URL (if present, expected only for outgoing
+           transactions).  So we cut the string off at the space. */
+        *space = '\0';
+      }
+      /* NOTE: For a real bank, the subject should include a checksum! */
+      if (GNUNET_OK !=
+          GNUNET_STRINGS_string_to_data (subject,
+                                         strlen (subject),
+                                         &wd.wtid,
+                                         sizeof (wd.wtid)))
+      {
+        /* Ill-formed wire subject, set binary version to all zeros
+           and pass as a string, this time including the part after
+           the space. */
+        memset (&wd.wtid,
+                0,
+                sizeof (wd.wtid));
+        wd.wtid_s = details->wire_transfer_subject;
+      }
+      else
+      {
+        wd.wtid_s = NULL;
+      }
       GNUNET_free (subject);
-      /* NOTE: for a "real" bank, we would want to trigger logic to undo the
-         wire transfer. However, for the "demo" bank, it should currently
-         be "impossible" to do wire transfers with invalid subjects, and
-         equally we thus don't need to undo them (and there is no API to do
-         that nicely either right now). So we don't handle this case for now. */
-      return;
-    }
-    GNUNET_free (subject);
-    wd.account_details = details->account_details;
+      wd.account_details = details->account_details;
 
-    if ( (NULL != whh->hres_cb) &&
-	 (GNUNET_OK !=
-	  whh->hres_cb (whh->hres_cb_cls,
-			dir,
-			&bserial_id,
-			sizeof (bserial_id),
-			&wd)) )
-      whh->hres_cb = NULL;
-  }
-  else
-  {
+      GNUNET_break (NULL != whh->hh);
+
+      /* Once we get the sentinel element, the handle becomes invalid. */
+      if (TALER_BANK_DIRECTION_NONE == dir)
+        whh->hh = NULL;
+
+      if ( (NULL != whh->hres_cb) &&
+           (GNUNET_OK !=
+            whh->hres_cb (whh->hres_cb_cls,
+                          TALER_EC_NONE,
+                          dir,
+                          &bserial_id,
+                          sizeof (bserial_id),
+                          &wd)) )
+        whh->hres_cb = NULL;
+      break;
+    }
+  case MHD_HTTP_NO_CONTENT:
     if (NULL != whh->hres_cb)
       (void) whh->hres_cb (whh->hres_cb_cls,
+                           ec,
                            TALER_BANK_DIRECTION_NONE,
                            NULL,
                            0,
                            NULL);
-    whh->hh = NULL;
     GNUNET_free (whh);
+    break;
+  default:
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Bank failed with HTTP status %u (EC: %u)\n",
+                http_status,
+                ec);
+    if (NULL != whh->hres_cb)
+      (void) whh->hres_cb (whh->hres_cb_cls,
+                           ec,
+                           TALER_BANK_DIRECTION_NONE,
+                           NULL,
+                           0,
+                           NULL);
+    GNUNET_free (whh);
+    break;
   }
 }
 
@@ -925,8 +961,14 @@ test_get_history (void *cls,
   if ( (NULL != start_off) &&
        (sizeof (uint64_t) != start_off_len) )
   {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Wire plugin 'test' got start offset of wrong size (%llu instead of %llu)\n",
+                (unsigned long long) start_off_len, (unsigned long long) sizeof (uint64_t));
     GNUNET_break (0);
-    return NULL;
+    /* Probably something is wrong with the DB, some other component
+     * wrote a wrong value to it.  Instead of completely stopping to work, 
+     * we just scan from the beginning. */
+    start_off = NULL;
   }
   if (NULL == start_off)
   {
@@ -942,7 +984,7 @@ test_get_history (void *cls,
   whh->hres_cb = hres_cb;
   whh->hres_cb_cls = hres_cb_cls;
   whh->hh = TALER_BANK_history (tc->ctx,
-                                tc->bank_uri,
+                                tc->bank_url,
                                 &tc->auth,
                                 (uint64_t) tc->exchange_account_no,
                                 direction,
@@ -956,6 +998,7 @@ test_get_history (void *cls,
     GNUNET_free (whh);
     return NULL;
   }
+
   return whh;
 }
 
@@ -970,11 +1013,138 @@ static void
 test_get_history_cancel (void *cls,
 			 struct TALER_WIRE_HistoryHandle *whh)
 {
-  TALER_BANK_history_cancel (whh->hh);
+  if (NULL != whh->hh)
+  {
+    TALER_BANK_history_cancel (whh->hh);
+    whh->hh = NULL;
+  }
   GNUNET_free (whh);
 }
 
 
+/**
+ * Context for a rejection operation.
+ */
+struct TALER_WIRE_RejectHandle
+{
+  /**
+   * Function to call with the result.
+   */
+  TALER_WIRE_RejectTransferCallback rej_cb;
+
+  /**
+   * Closure for @e rej_cb.
+   */
+  void *rej_cb_cls;
+
+  /**
+   * Handle for the reject operation.
+   */
+  struct TALER_BANK_RejectHandle *brh;
+};
+
+
+/**
+ * Callbacks of this type are used to serve the result of asking
+ * the bank to reject an incoming wire transfer.
+ *
+ * @param cls closure
+ * @param http_status HTTP response code, #MHD_HTTP_NO_CONTENT (204) for successful status request;
+ *                    #MHD_HTTP_NOT_FOUND if the rowid is unknown;
+ *                    0 if the bank's reply is bogus (fails to follow the protocol),
+ * @param ec detailed error code
+ */
+static void
+reject_cb (void *cls,
+           unsigned int http_status,
+           enum TALER_ErrorCode ec)
+{
+  struct TALER_WIRE_RejectHandle *rh = cls;
+
+  rh->brh = NULL;
+  rh->rej_cb (rh->rej_cb_cls,
+              ec);
+  GNUNET_free (rh);
+}
+
+
+/**
+ * Reject an incoming wire transfer that was obtained from the
+ * history. This function can be used to transfer funds back to
+ * the sender if the WTID was malformed (i.e. due to a typo).
+ *
+ * Calling `reject_transfer` twice on the same wire transfer should
+ * be idempotent, i.e. not cause the funds to be wired back twice.
+ * Furthermore, the transfer should henceforth be removed from the
+ * results returned by @e get_history.
+ *
+ * @param cls plugin's closure
+ * @param start_off offset of the wire transfer in plugin-specific format
+ * @param start_off_len number of bytes in @a start_off
+ * @param rej_cb function to call with the result of the operation
+ * @param rej_cb_cls closure for @a rej_cb
+ * @return handle to cancel the operation
+ */
+static struct TALER_WIRE_RejectHandle *
+test_reject_transfer (void *cls,
+                      const void *start_off,
+                      size_t start_off_len,
+                      TALER_WIRE_RejectTransferCallback rej_cb,
+                      void *rej_cb_cls)
+{
+  struct TestClosure *tc = cls;
+  const uint64_t *rowid_b64 = start_off;
+  struct TALER_WIRE_RejectHandle *rh;
+
+  if (sizeof (uint64_t) != start_off_len)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  rh = GNUNET_new (struct TALER_WIRE_RejectHandle);
+  rh->rej_cb = rej_cb;
+  rh->rej_cb_cls = rej_cb_cls;
+  rh->brh = TALER_BANK_reject (tc->ctx,
+                               tc->bank_url,
+                               &tc->auth,
+                               (uint64_t) tc->exchange_account_no,
+                               GNUNET_ntohll (*rowid_b64),
+                               &reject_cb,
+                               rh);
+  if (NULL == rh->brh)
+  {
+    GNUNET_free (rh);
+    return NULL;
+  }
+  return rh;
+}
+
+
+/**
+ * Cancel ongoing reject operation.  Note that the rejection may still
+ * proceed. Basically, if this function is called, the rejection may
+ * have happened or not.  This function is usually used during shutdown
+ * or system upgrades.  At a later point, the application must call
+ * @e reject_transfer again for this wire transfer, unless the
+ * @e get_history shows that the wire transfer no longer exists.
+ *
+ * @param cls plugins' closure
+ * @param rh operation to cancel
+ * @return closure of the callback of the operation
+ */
+static void *
+test_reject_transfer_cancel (void *cls,
+                             struct TALER_WIRE_RejectHandle *rh)
+{
+  void *ret = rh->rej_cb_cls;
+
+  if (NULL != rh->brh)
+    TALER_BANK_reject_cancel (rh->brh);
+  GNUNET_free (rh);
+  return ret;
+}
+
+
 /**
  * Initialize test-wire subsystem.
  *
@@ -996,12 +1166,12 @@ libtaler_plugin_wire_test_init (void *cls)
     if (GNUNET_OK !=
         GNUNET_CONFIGURATION_get_value_string (cfg,
                                                "exchange-wire-test",
-                                               "BANK_URI",
-                                               &tc->bank_uri))
+                                               "BANK_URL",
+                                               &tc->bank_url))
     {
       GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                  "exchange-wire-test",
-                                 "BANK_URI");
+                                 "BANK_URL");
       GNUNET_free (tc);
       return NULL;
     }
@@ -1014,7 +1184,7 @@ libtaler_plugin_wire_test_init (void *cls)
       GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                  "exchange-wire-test",
                                  "EXCHANGE_ACCOUNT_NUMBER");
-      GNUNET_free (tc->bank_uri);
+      GNUNET_free (tc->bank_url);
       GNUNET_free (tc);
       return NULL;
     }
@@ -1027,7 +1197,7 @@ libtaler_plugin_wire_test_init (void *cls)
       GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                  "taler",
                                  "CURRENCY");
-      GNUNET_free (tc->bank_uri);
+      GNUNET_free (tc->bank_url);
       GNUNET_free (tc);
       return NULL;
     }
@@ -1040,7 +1210,7 @@ libtaler_plugin_wire_test_init (void *cls)
       GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                  "exchange-wire-test",
                                  "USERNAME");
-      GNUNET_free (tc->bank_uri);
+      GNUNET_free (tc->bank_url);
       GNUNET_free (tc);
       return NULL;
     }
@@ -1053,7 +1223,7 @@ libtaler_plugin_wire_test_init (void *cls)
       GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                  "exchange-wire-test",
                                  "PASSWORD");
-      GNUNET_free (tc->bank_uri);
+      GNUNET_free (tc->bank_url);
       GNUNET_free (tc);
       GNUNET_free (user);
       return NULL;
@@ -1068,7 +1238,7 @@ libtaler_plugin_wire_test_init (void *cls)
     {
       GNUNET_break (0);
       GNUNET_free (tc->currency);
-      GNUNET_free (tc->bank_uri);
+      GNUNET_free (tc->bank_url);
       GNUNET_free (tc->auth.details.basic.username);
       GNUNET_free (tc->auth.details.basic.password);
       GNUNET_free (tc);
@@ -1087,6 +1257,8 @@ libtaler_plugin_wire_test_init (void *cls)
   plugin->execute_wire_transfer_cancel = &test_execute_wire_transfer_cancel;
   plugin->get_history = &test_get_history;
   plugin->get_history_cancel = &test_get_history_cancel;
+  plugin->reject_transfer = &test_reject_transfer;
+  plugin->reject_transfer_cancel = &test_reject_transfer_cancel;
   return plugin;
 }
 
@@ -1131,7 +1303,7 @@ libtaler_plugin_wire_test_done (void *cls)
     break;
   }
   GNUNET_free_non_null (tc->currency);
-  GNUNET_free_non_null (tc->bank_uri);
+  GNUNET_free_non_null (tc->bank_url);
   GNUNET_free (tc);
   GNUNET_free (plugin);
   return NULL;
